/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @author JcHartmut: hartmut.schorrig@vishia.de
 * @version 0.82
 * list of changes:
 * 2008-04-06 JcHartmut: some correction
 * 2008-03-15 JcHartmut: creation
 *
 ****************************************************************************/
package org.vishia.java2C;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.vishia.zbnf.ZbnfParseResultItem;

/**This class processes the first pass of translating Java to C of a detect class in a java file.
 * Parsing the content of the java file is done before, see {@link GenerateFile}.
 * In the first pass all declared elements of a class are captured and store in the {@link ClassData}.  
 * 
 * @author JcHartmut
 *
 */
public class FirstPass extends GenerateClass
{

  /**initializes the instances with the given associations. */ 
  //FirstPass(iWriteContent writeContent, TreeMap<String, ClassData> pkgIdents, RunRequiredFirstPass_ifc runRequiredFirstPass)
  FirstPass(iWriteContent writeContent, GenerateFile parentGenerateFile, LocalIdents fileLevelIdents, RunRequiredFirstPass_ifc runRequiredFirstPass)
  { super(writeContent, parentGenerateFile, fileLevelIdents, runRequiredFirstPass);
  }


  /**Generates the content of a class in the h-file with given ZBNF parse result item of the class.
   * This routine is called from {@link GenerateFile}.  
   * <ul>
   * <li> All internal classes and interfaces are detect, this method is called recursively with it.
   * <li>The class fields are written in a struct in the h-File. 
   * <li> The static fields are written in h-file as extern declaration.
   * <li> The constructors are written as forward declaration in h-file of routines ctor...
   * <li> The methods are written as forward declaration in h-file.
   * </ul> 
   * For fields, the initial values are detected and stored in {@link ClassData#variablesToInit}.
   * This action is done inside the called routine {@link GenerateClass#gen_variableDefinition(ZbnfParseResultItem, LocalIdents, List, char, ClassData[])}.
   * <br>
   * The method heads are generated calling {@link #write_methodDeclaration(ZbnfParseResultItem, String, LocalIdents)}. 
   * 
   * @param fileContainsClass The generation environment
   * @param sFileName Name of the file for C without extension .c or .h, not the Java-filename
   *        It is stored in the created ClassData of the class
   *        to allow the generating of a include statement, if another class uses this class.
   * @param sPkgIdent Package where the Java file is found. With <code>/</code> as Separator and at end.
   * @param sPrefix To Build the Classname for C
   * @param sPostfix To Build the Classname for C.
   * @param zbnfClass parse result of <code>class::=</code>
   * @param sOuterClassName "" or the name of the outer class. 
   *        The name of the outer class is set as prefix of the name of the struct in C
   * @param outerClassData The outer class data or null, to register the inner class in a outer.
   * @param listAllClassesForSecondPath List to add the detect ClassData. 
   *        The list is used as input for running the second path of classes. 
   *        See {@link GenerateFile.itemsClassesForSecondPath}.  
   * @param bInterface
   * @param identsParent
   * @return
   * @throws IOException
   * @throws ParseException
   * @throws IllegalArgumentException
   * @throws IllegalAccessException
   * @throws InstantiationException
   */
  ClassData runFirstPassClass
  ( GenerateFile fileContainsClass
  , String sFileNameC
  , String sPkgIdent    
  , String sPrefixClassCname, String sSuffixClassCname
  , ZbnfParseResultItem zbnfClass
  , String sOuterClassName
  , ClassData outerClassData
  , List<ClassData> listAllClassesForSecondPath
  , boolean bInterface
  )
  throws IOException, ParseException, IllegalArgumentException, IllegalAccessException, InstantiationException
  {
    ZbnfParseResultItem itemClassDescription = zbnfClass.getChild("description");
    boolean bStaticInstance =  itemClassDescription != null
                            && itemClassDescription.getChild("staticInstance")!= null;
    boolean bFinal = zbnfClass.getChild("final")!= null;
    ZbnfParseResultItem itemClassIdent = zbnfClass.getChild("classident");
    if(itemClassIdent == null)
      throw new ParseException("classident not detect", 0);
    final String sClassNameLocal = itemClassIdent.getParsedString();
    if(sClassNameLocal.equals("LogMessageFile"))
      stop();
    final String sClassNameC;
    final String sClassNameJava;
    
    final ClassData superClazz = zbnfResultSuperclass(zbnfClass, itemClassDescription, outerClassData);
    ClassData[] ifcClassData = zbnfResultInterfaces(zbnfClass, itemClassDescription, outerClassData);
    
    if(outerClassData != null)
    { sClassNameC = outerClassData.getClassIdentName() + "__" + sClassNameLocal;
      //sClassNameJava = outerClassData.sClassNameJava + "." + sClassNameLocal;
      sClassNameJava = sClassNameLocal;  
      //NOTE: use sClassName for classData from outer class "__" + read from itemClassIdent 
      classData = new ClassData(fileContainsClass, sFileNameC, sPkgIdent, sClassNameJava, sClassNameC + "_s"
          , "XX", 'L', outerClassData, superClazz, ifcClassData, zbnfClass
          , bStaticInstance, bFinal, bInterface, fileLevelIdents);
      /**An inner class is known in the outer by name. */
      outerClassData.classLevelIdents.putClassType(sClassNameLocal, classData);
    }
    else
    { //NOTE: use sClassName for classData only read from itemClassIdent 
      sClassNameC = sPrefixClassCname + (sSuffixClassCname != null ? sClassNameLocal + sSuffixClassCname : "");
      sClassNameJava = sClassNameLocal;
      classData = new ClassData(fileContainsClass, sFileNameC, sPkgIdent, sClassNameJava, sClassNameC + "_s"
          , "XX", 'L', null, superClazz, ifcClassData, zbnfClass
          , bStaticInstance, bFinal, bInterface, fileLevelIdents);
      /** TODO put in list of this package.
       * A non-inner class is known in the package by name. Store it to find out later in package by name. */
      fileLevelIdents.putClassType(sClassNameLocal, classData); 
      /**A non-inner class is known globally by pkg/name. Store it to find out later globally by package/name.*/
      //fileLevelIdents.putClassType(sPkgIdent + sClassNameLocal, classData);
      
    }
    
    /**Register the class just now, to prevent circular call of first pass 
     * if an one-another internal usage of a class as reference is programmed in Java. */
    fileContainsClass.registerClassInFile(classData);
    
    //pkgIdents.put(sClassName, classData);   //to find it later sorted by names.
    listAllClassesForSecondPath.add(classData);  //to process in the second path sorted per file
    if(sClassNameLocal.equals("ExpandedDataStruct"))
      stop();
    //setInitialTypeIdentifiers();

    { /**First process subclasses because there may be known before the outer class is translated. */
      List<ZbnfParseResultItem> listSubClasses = zbnfClass.listChildren("classDefinition");
      if(listSubClasses != null)
      { for(ZbnfParseResultItem itemSubClasses : listSubClasses)
        { FirstPass genClass = new FirstPass(writeContent, fileContainsClass, fileLevelIdents, runRequiredFirstPass);
          genClass.runFirstPassClass(fileContainsClass, sFileNameC, sPkgIdent, null, null
              , itemSubClasses, sClassNameLocal + "__", classData , listAllClassesForSecondPath
              , false);
      } }
    }
    { List<ZbnfParseResultItem> listSubInterfaces = zbnfClass.listChildren("interfaceDefinition");
      if(listSubInterfaces != null)
      { for(ZbnfParseResultItem itemSubInterfaces : listSubInterfaces)
        { write_Interface(itemSubInterfaces, sClassNameLocal + "__", classData);
      } }
    }

    //inner classes will be written before, now the own class: 
    write_HeaderContent(fileContainsClass, zbnfClass, sClassNameC, superClazz, ifcClassData);
    //
    if(sClassNameJava.equals("SimpleClass"))
      stop();
    return classData;
  }

  
  

  /**Gets the superClass from ZBNF parse result item.
   * @param zbnfClass The parse result item of <code>classDefinition::=...</code>
   * @param zbnfDescription The parse result item of <code>< description>... test_description::=</code>
   *        inside the <code>classDefinition::=...</code>  
   * @param outerClassData a given outerclass
   * @return The ClassData of the superClass. It is {@link Java2C_Main.CRuntimeJavalikeClassData#clazzObjectJc}
   *         if no superclass is given in Java-code (no <code>extends ...</code>). 
   *         It is null if the description contains <code>@java2c=noObject.</code> and no other superclass is given.
   * @throws FileNotFoundException The Superclass may be translated yet nested, it may be cause some errors. 
   * @throws IllegalArgumentException Errors see {@link #runFirstPassClass(GenerateFile, String, String, String, String, ZbnfParseResultItem, String, ClassData, List, boolean, LocalIdents)}.
   * @throws IOException
   * @throws IllegalAccessException
   * @throws InstantiationException
   * @throws ParseException 
   */
  ClassData zbnfResultSuperclass(ZbnfParseResultItem zbnfClass, ZbnfParseResultItem zbnfDescription, ClassData outerClassData) 
  throws FileNotFoundException, IllegalArgumentException, IOException, IllegalAccessException, InstantiationException, ParseException
  { final ClassData superClazz;
    boolean bNoObject =  zbnfDescription != null && zbnfDescription.getChild("noObject")!= null;
    if(bNoObject)
      stop();
    final ZbnfParseResultItem zbnfSuperClass = zbnfClass.getChild("Superclass");
    if(zbnfSuperClass != null)
    { String sSuperClass = zbnfSuperClass.getParsedString();
      stop();
      if(outerClassData != null)
      { /*If there is an outer class, the superclass may be part of them or may be known in the file. */
        //superClazz = outerClassData.classLevelIdents.getTypeContingentlyRunFirstPass(sSuperClass, runRequiredFirstPass);
        superClazz = outerClassData.classLevelIdents.getType(sSuperClass, fileLevelIdents);
      }
      else {
        //superClazz = fileLevelIdents.getTypeContingentlyRunFirstPass(sSuperClass, runRequiredFirstPass);
        superClazz = fileLevelIdents.getType(sSuperClass, null);
      }
    }
    else
    {
      superClazz = bNoObject ? null : CRuntimeJavalikeClassData.clazzObjectJc;
    }
    return superClazz;
  }  

  
  
  
  
  /**Gets the interfaces from ZBNF parse result item.
   * @param zbnfClass The parse result item of <code>classDefinition::=...</code>
   * @param zbnfDescription The parse result item of <code>< description>... test_description::=</code>
   *        inside the <code>classDefinition::=...</code>  
   * @param outerClassData a given outerclass
   * @return Array of ClassData[] of all Interfaces. If the class hasn't interfaces, it is null.
   * @throws FileNotFoundException The Superclass may be translated yet nested, it may be cause some errors. 
   * @throws IllegalArgumentException Errors see {@link #runFirstPassClass(GenerateFile, String, String, String, String, ZbnfParseResultItem, String, ClassData, List, boolean, LocalIdents)}.
   * @throws IOException
   * @throws IllegalAccessException
   * @throws InstantiationException
   * @throws ParseException 
   */
  private ClassData[] zbnfResultInterfaces(ZbnfParseResultItem zbnfClass, ZbnfParseResultItem zbnfDescription, ClassData outerClassData) 
  throws FileNotFoundException, IllegalArgumentException, IOException, IllegalAccessException, InstantiationException, ParseException
  { final ClassData[] interfaceClazzes;
    final List<ZbnfParseResultItem> zbnflistInterfaces = zbnfClass.listChildren("ImplementedInterface");
    if(zbnflistInterfaces != null){
      interfaceClazzes = new ClassData[zbnflistInterfaces.size()];
      int ixIfc = 0;
      for(ZbnfParseResultItem zbnfInterface: zbnflistInterfaces)
      { String sIdentInterface = zbnfInterface.getParsedString();
        if(outerClassData != null)
        { /*If there is an outer class, the superclass may be part of them or may be known in the file. */
          //interfaceClazzes[ixIfc] = outerClassData.classLevelIdents.getTypeContingentlyRunFirstPass(sIdentInterface, runRequiredFirstPass);
          interfaceClazzes[ixIfc] = outerClassData.classLevelIdents.getType(sIdentInterface, fileLevelIdents);
        }
        else {
          //interfaceClazzes[ixIfc] = fileLevelIdents.getTypeContingentlyRunFirstPass(sIdentInterface, runRequiredFirstPass);
          interfaceClazzes[ixIfc] = fileLevelIdents.getType(sIdentInterface, null);
        }
        ixIfc +=1;
      }
    }
    else
    {
      interfaceClazzes = null;
    }
    return interfaceClazzes;
  }  

  
  
  
  
  /**Write out the header part of the class.
   * It writes:
   * <pre>
   * /*@CLASS_C NAME @@@@@@@@@@@@@@@@@@@@@* /
   * typedef struct NAME_t
   * { union base_{ ObjectJc object; SUPER super; IFC IFC; } base
   *   VARIABLETYPE VARIABLENAME;
   * } NAME_s;
   * 
   * typedef struct NAME_Y_t { ObjectArrayJc head; " + NAME + "_s data[50]; } NAME_Y;");
   * 
   * extern struct ClassJc_t const reflection_NAME_s;
   *
   * extern STATICVARIABLEDEF NAME;
   * 
   * ...etc TODO  
   * </pre>  
   * @param fileContainsClass The calling instance.
   * @param zbnfClass The parse result item of the class definition.
   * @param sClassNameC The name, it is dissolved already from zbnfClass
   * @param superClazz The superclass or ObjectJc or null
   * @param ifcClassData null or interfaces
   * @throws FileNotFoundException
   * @throws IllegalArgumentException
   * @throws ParseException
   * @throws IOException
   * @throws IllegalAccessException
   * @throws InstantiationException
   */
  private void write_HeaderContent
  ( GenerateFile fileContainsClass
  , ZbnfParseResultItem zbnfClass    
  , String sClassNameC
  , ClassData superClazz
  , ClassData[] ifcClassData
  ) 
  throws FileNotFoundException, IllegalArgumentException, ParseException, IOException, IllegalAccessException, InstantiationException
  { final boolean bInterface = classData.isInterface();
    writeContent.writeClassH("\n\n/*@CLASS_C " + sClassNameC + " @@@@@@@@@@@@@@@@@@@@@@@@*/");
    writeContent.writeClassH("\n\ntypedef struct " + sClassNameC + "_t\n{ ");
    ClassData superClass = classData.getSuperClassData();
    if(superClass != null || ifcClassData != null)
    { writeContent.writeClassH("\n  union { ");
      final String sNameSuper; sNameSuper = (superClass).sClassNameC;
      if(superClass.isBasedOnObject() || sNameSuper.equals("ObjectJc"))
      { /**either the superclass based on or is an Object: */
        writeContent.writeClassH("ObjectJc object; ");
      }
      if(!sNameSuper.equals("ObjectJc"))
      { writeContent.writeClassH(sNameSuper + " super;");
        fileContainsClass.addIncludeH(superClazz.sFileName, "superclass");
      }
      if(ifcClassData != null) for(ClassData ifc: ifcClassData)
      { String sNameIfc_s = ifc.sClassNameC;
        String sNameIfc = ifc.getClassIdentName();
        writeContent.writeClassH(sNameIfc_s+ " " + sNameIfc +";");  //Type and name are identical
        fileContainsClass.addIncludeH(ifc.sFileName, "interface");
      }
      writeContent.writeClassH("} base; ");
    }
    else
      stop();
    
    if(sClassNameC.equals("ImplIfc_TestAll"))
      stop();
    //process the class Content, write all class variable in the struct
    List<ZbnfParseResultItem> listVariables = zbnfClass.listChildren("variableDefinition");
    if(listVariables != null)for(ZbnfParseResultItem itemVariable : listVariables)
    { if(itemVariable.getChild("static")== null)
      { //only non static members.
        { String sVariableName = itemVariable.getChild("variableName").getParsedString();
          if(sVariableName.equals("targets"))
            stop();
        }
        ZbnfParseResultItem zbnfVariableDescription = itemVariable.getChild("description"); //may be null.
        CCodeData sDef = gen_variableDefinition(itemVariable, zbnfVariableDescription, classData.classLevelIdents, classData.getVariablesToInit(), 'c');
        writeContent.writeClassH("\n  " + sDef.cCode);
      }  
    }
    
    writeContent.writeClassH("\n} " + sClassNameC + "_s;\n  \n");


    writeContent.writeClassH("\ntypedef struct " + sClassNameC + "_Y_t { ObjectArrayJc head; " + sClassNameC + "_s data[50]; } " + sClassNameC + "_Y;");
    //writeContent.writeClassH("\ntypedef struct " + sClassNameC + "_Y_t { ObjectJc object; int32 length; int16 sizeElement; int16 mode; " + sClassNameC + "_s data[50]; } " + sClassNameC + "_Y;");
    
    writeContent.writeClassH("\n\nextern struct ClassJc_t const reflection_" + sClassNameC + "_s;\n  \n");

    
    if(listVariables != null)for(ZbnfParseResultItem itemVariable : listVariables)
    { if(itemVariable.getChild("static")!= null)
      { //only non static members.
        ZbnfParseResultItem zbnfVariableDescription = itemVariable.getChild("description"); //may be null.
        CCodeData sDef = gen_variableDefinition(itemVariable, zbnfVariableDescription, classData.classLevelIdents, classData.getVariablesToInit(), 's');
        if(sDef.identInfo.modeStatic=='d'){
          writeContent.writeClassH("\n" + sDef.cCode);
        }else{
          writeContent.writeClassH("\nextern " + sDef.cCode);
        }
      }  
    }
    writeContent.writeClassH("\n\n");
    
    if(!bInterface)
    { final String nameCtor = classData.isBasedOnObject()?  "ctorO" : "ctorM";
      List<ZbnfParseResultItem> listConstructors = zbnfClass.listChildren("constructorDefinition");
      if(listConstructors != null)
      { boolean bArgumentSensitive = (listConstructors.size() >1);
        for(ZbnfParseResultItem itemConstructor : listConstructors)
        { Method method = gen_methodHeadAndRegisterMethod(itemConstructor, true, classData, nameCtor, classData.classTypeInfo, bArgumentSensitive);
          writeContent.writeClassH(method.gen_MethodForwardDeclaration());
          ClassData.MethodWithZbnfItem methodDef = classData.new MethodWithZbnfItem(method, itemConstructor, 'c'); 
          classData.listMethodWithZbnf.add(methodDef);
        }
      }
      else
      { //no constructor found, create a default constructor because all variable initializations should be done.
        Method method = gen_methodHeadAndRegisterMethod(null, true, classData, nameCtor, classData.classTypeInfo, false);
        writeContent.writeClassH(method.gen_MethodForwardDeclaration());
        ClassData.MethodWithZbnfItem methodDef = classData.new MethodWithZbnfItem(method, null, 'c'); 
        classData.listMethodWithZbnf.add(methodDef);
      }
      //writeContent.writeClassH("\n\n" + "void finalize_" + sClassNameC + "(ObjectJc* yObj, ThCxt* _thCxt);\n\n");
    }
  
    if(bInterface)
      stop();
    
    { List<ZbnfParseResultItem> listMethods = zbnfClass.listChildren("methodDefinition");
      if(listMethods != null)
      { 
        for(ZbnfParseResultItem itemMethod : listMethods)
        { String sName = itemMethod.getChild("name").getParsedString();
          //register the method with following "?" to test if it is ambiguous.
          classData.testAndSetAmbiguousnessOfMethod(sName);
        } 
        for(ZbnfParseResultItem itemMethod : listMethods)
        { write_methodDeclaration(itemMethod, sClassNameC, classData.classLevelIdents);
        } 
      }
    }
    if(classData.isFinalizeNeed()){
      /**Finalize is not written in Java, but need because enhanced references: */
      classData.addMethod("finalize", "finalize", Method.modeOverrideable, CRuntimeJavalikeClassData.clazz_void.classTypeInfo);
    }
    if(classData.inheritanceInfo != null){
      classData.completeInheritanceWithOwnMethods();
    }  
    write_Mtbl();
    
  }  
  
  
  
  private void write_Mtbl()
  {  
    String sClassIdentName = classData.getClassIdentName();
    String sClassName_s = classData.getClassCtype_s();
    boolean isObject;
    writeContent.writeClassH("\n\n/* J2C: Method table contains all dynamic linked (virtual) methods");      
    writeContent.writeClassH("\n * of the class and all super classes and interfaces. */");      
    writeContent.writeClassH("\nextern const char sign_Mtbl_" + sClassIdentName + "[]; //marker for methodTable check");
    writeContent.writeClassH("\ntypedef struct Mtbl_" + sClassIdentName + "_t");      
    writeContent.writeClassH("\n{ ");      
    String sMtbl = classData.gen_MethodTableDefinitionContent();  //from this class and all its super classes and interfaces.
    writeContent.writeClassH(sMtbl);
    writeContent.writeClassH("\n} Mtbl_" + sClassIdentName + ";\n\n");      
  }  
  
  
  
  
  
  /**Generates the content of an interface in c- and h-file. 
   * @throws InstantiationException 
   * @throws IllegalAccessException 
   * @throws IllegalArgumentException */
  void write_Interface(ZbnfParseResultItem itemClass, String sOuterClassName, ClassData outerClassData)
  throws IOException, ParseException, IllegalArgumentException, IllegalAccessException, InstantiationException
  {
    //ZbnfParseResultItem resultItem = resultItemClass.nextSkipIntoComponent(resultItemClass);

    ZbnfParseResultItem itemClassIdent = itemClass.getChild("interfaceIdent");
    if(itemClassIdent == null) throw new ParseException("interfaceIdent not detect", 0);

    ZbnfParseResultItem itemDescription = itemClass.getChild("description");
    if(itemDescription != null)
    { write_Description(itemDescription);
    }


    String sClassName = sOuterClassName + itemClassIdent.getParsedString();
    writeContent.writeClassH("typedef struct " + sClassName + "_t  /*interface*/\n{");
    writeContent.writeClassH("ObjectJc object;");

    if(sClassName == null) throw new ParseException("no className found", 0);

    Iterator<ZbnfParseResultItem> iterMethod = itemClass.iterChildren("methodDefinition");
    if(iterMethod != null)
    { while(iterMethod.hasNext())
      { ZbnfParseResultItem itemMethod = iterMethod.next();
        write_methodDeclaration(itemMethod, sClassName, classData.classLevelIdents);
    } }

    writeContent.writeClassH("\n} " + sClassName + "_i;\n  \n");
    
    List<ZbnfParseResultItem> listSubClasses = itemClass.listChildren("classDefinition");
    if(listSubClasses != null)
    { for(ZbnfParseResultItem itemSubClasses : listSubClasses)
      { //runFirstPass(itemSubClasses, sClassName + "__", itemsxxx);
      }
    }
  }



  /**writes a method-forward-declaration in Headerfile and registers the method to generate definition.
   * It calls 
   *
   * @param zbnfMethod  with syntax methodDefinition::=.
   * @throws IOException
   * @throws ParseException
   * @throws InstantiationException 
   * @throws IllegalAccessException 
   * @throws IllegalArgumentException 
   */
  public void write_methodDeclaration(ZbnfParseResultItem zbnfMethod, String sClassName, LocalIdents localIdents)
  throws IOException, ParseException, IllegalArgumentException, IllegalAccessException, InstantiationException
  { String sName = zbnfMethod.getChild("name").getParsedString();
    if(sName.equals("provideOutput"))
      stop();
    //if(!sName.equals("finalize"))  //NOTE: finalize Method see write_finalizeDefinition()
    {    
      //Method method = classData.searchMethod(sName + "?", null);
      boolean bArgumentSensitive = classData.isAmbiguousnessMethod(sName); //method.isAmbigous();
      ZbnfParseResultItem itemType = zbnfMethod.getChild("type");
      ZbnfParseResultItem zbnfDescription = zbnfMethod.getChild("description");
      ZbnfParseResultItem zbnfReturnDescription = zbnfDescription == null ? null : zbnfDescription.getChild("returnDescription");
      FieldData methodType = getTypeInfo(null, itemType, zbnfReturnDescription, localIdents, null, 'a'); 
      Method method = gen_methodHeadAndRegisterMethod(zbnfMethod, false, classData, sName, methodType, bArgumentSensitive);
      if(sName.equals("finalize")){
        /**NOTE: finalize Method see write_finalizeDefinition()*/
        classData.setBodyForFinalize(zbnfMethod);
      }
      else {
        writeContent.writeClassH(method.gen_MethodForwardDeclaration());
      }
      ClassData.MethodWithZbnfItem methodDef = classData.new MethodWithZbnfItem(method, zbnfMethod, 'm'); 
      classData.listMethodWithZbnf.add(methodDef);
             
    }
  }


  

  /**Generates a method head with syntax <code>methodDefinition::=</code> from given ZBNF parse result item
   * and registers the method in the classData. The ambiguousness of the method should be tested before this call,
   * but the unambiguous method name for C is built here: Methods with equal names but different parameter types 
   * are different methods.
   * 
   * @param zbnfMethod Zbnf item defined in <code>&lt;methodDefinition></code> 
   *        or <code>&lt;constructorDefinition></code>. The head hasn't an own zbnf-result item.
   *        The following contained elements are tested:
   *        <ul>
   *        <li><code>&lt;description></code>, calling {@link #get_shortDescription(ZbnfParseResultItem)}
   *        <li><code>&lt;static></code> defined in <code>Modifier::=<?></code> 
   *        <li><code>&lt;argumentList></code>, calling {@link #gen_methodArguments(ZbnfParseResultItem, LocalIdents, char, String[], List)}
   *        </ul>
   *        If this argument is null, a default constructor is generated.
   * @param classOfMethod environment class
   * @param sMethodNameJava Name of the method, "ctorO" if a constructor from given ObjectJc should be generated.
   * @param typeReturn Type of the method, get outside because this routine is also used to generate constructors.
   *        On constructor generation this argument should be equal the environment class type. 
   * @param bArgumentSensitive true than more as one method with the same Java-name exists. 
   *        It is tested before calling {@link ClassData#isAmbiguousnessMethod(String)}.
   *        The C-name of the method is built regarding the type of the arguments.
   * @return The created Method. It contains the head definition in one line in its private attribute 
   *         {@link Method#sMethodFormalListDefiniton} 
   *         promoted in {@link Method#gen_MethodForwardDeclaration()} and 
   *         {@link Method#gen_MethodHeadDefinition()}. 
   * @throws IOException
   * @throws ParseException
   * @throws InstantiationException 
   * @throws IllegalAccessException 
   * @throws IllegalArgumentException 
   */
  protected Method gen_methodHeadAndRegisterMethod
  ( ZbnfParseResultItem zbnfMethod
  , boolean bCtor
  //, String sClassName
  , ClassData classOfMethod    
  , String sMethodNameJava
  , FieldData typeReturn
  , boolean bArgumentSensitive
  )
  throws IOException, ParseException, IllegalArgumentException, IllegalAccessException, InstantiationException
  { LocalIdents methodIdents = new LocalIdents(classData.classLevelIdents);
    ZbnfParseResultItem zbnfDescription = zbnfMethod == null ? null : zbnfMethod.getChild("description");
    String sDescription = zbnfMethod!=null ? get_shortDescription(zbnfDescription) : "Default constructor. ";
    assert(classOfMethod == classData);  //the argument classOfMethod is unnecessary.
    String sNameUnambiguous = sMethodNameJava;
    String sArguments;
    String sClassName = classOfMethod.getClassIdentName();
    String sMethodClassType_s = classOfMethod.getClassCtype_s();
    if(sMethodNameJava.equals("testIfc") && sClassName.equals("TestAllConcepts_Test"))
      stop();
    final boolean isStatic = zbnfMethod !=null && zbnfMethod.getChild("static") != null;
      
    final boolean bFinal = zbnfMethod ==null || zbnfMethod.getChild("final")!=null;  
    if(bFinal)
      stop();
    int modifier = 0;
    if(classData.isInterface() || (!bCtor && !bFinal && !isStatic)){
      /**dynamic call method. */
      modifier |= Method.modeOverrideable;
    }
    if(isStatic){
      modifier |= Method.modeStatic;
    }
    if(zbnfDescription != null){
      //if(zbnfDescription.getChild("override-able")!= null){ modifier |= Method.modeOverrideable;}
      if(zbnfDescription.getChild("return-this")!= null)  { modifier |= Method.modeReturnThis; }
      if(zbnfDescription.getChild("noStacktrace")!= null)  { modifier |= (Method.modeNoStacktrace | Method.modeNoThCxt); }
      if(zbnfDescription.getChild("noThCxt")!= null)  { modifier |= Method.modeNoThCxt; }
    }
    FieldData[] argTypes; // =   bArgumentSensitive ? new ClassData[30] : null; //more than 30 args are not expected.
    //List<ClassData> listArgTypes =   bArgumentSensitive ? new LinkedList<ClassData>() : null; //more than 30 args are not expected.
    List<FieldData> listArgTypes = new LinkedList<FieldData>(); //more than 30 args are not expected.
    if(zbnfMethod!=null) //not the default constructor
    { //NOTE: call without zbnfMethod if a default ctor is created.
      String[] sArgSensitive = bArgumentSensitive ? new String[1] : null;
      char intensionArgument = 'a'; //(intension == 'h') ? 'A' : 'a'; //declaration or definition list.
      sArguments = gen_methodArguments(zbnfMethod, methodIdents, intensionArgument, sArgSensitive, listArgTypes);
      if(bArgumentSensitive && sArgSensitive[0].length() >0)
      { sNameUnambiguous += "_" + sArgSensitive[0];
      }
      if(listArgTypes != null && listArgTypes.size() >0)
      { argTypes = new FieldData[listArgTypes.size()];
        listArgTypes.toArray(argTypes);
      }
      else
      { argTypes = null;
      }
    }
    else
    { sArguments = "";
      argTypes = null;
    }
    
    
    final Method methodData = classData.addMethod( sMethodNameJava, sNameUnambiguous 
      , modifier, typeReturn, argTypes);

    methodData.setMethodIdents(methodIdents);  //classLevelIdents plus arguments.
    String sTypeReturn = typeReturn.gen_Declaration();
    String sMethodHead = "(";  //use the really name.
    if(sMethodNameJava.equals("ctorO"))
    { sMethodHead += "ObjectJc* othis";
    }
    else if(sMethodNameJava.equals("ctorM"))
    { sMethodHead += "MemC mthis";
    }
    else if(methodData.firstDeclaringClass.isInterface())
    { /**An overridden method of an interface-declared method. */
      sMethodHead += "ObjectJc* ithis";  //Note: all interface methods are called with an Object-reference.
    }
    else if(methodData.firstDeclaringClass != classData)
    { /**An overridden method of an superclass-declared method. */
      sMethodHead += methodData.firstDeclaringClass.getClassCtype_s() + "* ithis";
    }
    else if(!isStatic)
    { sMethodHead += sMethodClassType_s + "* ythis";
    }
    else
    { sMethodHead += "/*static*/ ";
    }
    if(sArguments.length() >0){
      sMethodHead += (!isStatic ? ", " : "") + sArguments;
    }  
    if(methodData.need_thCxt){
      sMethodHead += (sArguments.length() >0 || !isStatic ? ", ": "") + "ThCxt* _thCxt)";
    }
    else{
      sMethodHead += ")";
    }
      
    methodData.setMethodHeadDefiniton(sDescription, sTypeReturn, sMethodHead);
    
    return methodData;
  }


  
  
  
  /**Generates the formal argument list of a method used both by generating method declaration or method definition.
   * @param zbnfMethod Zbnf item defined in <code>argumentList::=<?>...</code>.
   * @param localIdents The known identificators of the calling environment, the type identificators are used especially.
   *        This parameter is used calling {@link #gen_variableDefinition(ZbnfParseResultItem, LocalIdents, List, char intension, ClassData[])}
   *        inside with the parameters of the head. If the intension=='a' (Method definition in C-File),the parameter-variables are added to the localIdents.
   * @param intension Intension of call: 'A'-declaration in h-File, 'a'-definition in c-file, passing on <code>gen_variableDefinition()</code>. 
   * @param retArgSensitiveName [0] is filled with the short name of arg types if not null.
   * @param retArgTypes The list is filled with the detect argument types, used for argument sensitive method call.
   * @return String of formal arguments.
   * @throws ParseException
   * @throws InstantiationException 
   * @throws IllegalAccessException 
   * @throws IOException 
   * @throws IllegalArgumentException 
   * @throws FileNotFoundException 
   */
  String gen_methodArguments
  ( ZbnfParseResultItem zbnfMethod
  , LocalIdents localIdents
  , char intension
  , String[] retArgSensitiveName
  , List<FieldData> retArgTypes
  ) 
  throws ParseException, FileNotFoundException, IllegalArgumentException, IOException, IllegalAccessException, InstantiationException
  { final Map<String, ZbnfParseResultItem> indexParamDescription;
    ZbnfParseResultItem zbnfDescription = zbnfMethod.getChild("description");
    if(zbnfDescription != null)
    { final List<ZbnfParseResultItem> listZbnfParamDescription = zbnfDescription.listChildren("param");
      if(listZbnfParamDescription != null)
      { indexParamDescription = new TreeMap<String, ZbnfParseResultItem>();
        for(ZbnfParseResultItem zbnfParamDescription: listZbnfParamDescription)
        { String sName = zbnfParamDescription.getChild("variableName").getParsedString();
          indexParamDescription.put(sName, zbnfParamDescription);  
        }
      }
      else { indexParamDescription = null; }
    }
    else { indexParamDescription = null; }
    
    Iterator<ZbnfParseResultItem> iterParam = zbnfMethod.iterChildren("argument");
    boolean bArgumentSensitive = (retArgSensitiveName != null);
    String sArgs = "";
    String sArgSensitiveName = "";
    String sArgSeparator = "";
    if(iterParam != null)
    { byte idxArg = 0;
      boolean bFirst = true;
      while(iterParam.hasNext())
      { ZbnfParseResultItem zbnfParam = iterParam.next();
        String sName = zbnfParam.getChild("variableName").getParsedString();
        ZbnfParseResultItem zbnfParamDescription = indexParamDescription == null ? null : indexParamDescription.get(sName);
        if(zbnfParamDescription != null)
          stop();
        if(sName.equals("valueTEST"))
          stop();
        CCodeData codeArg = gen_variableDefinition(zbnfParam, zbnfParamDescription, localIdents, null, intension);
        //sArgs += sArgSeparator + sArg;
        if(bFirst){ 
          bFirst = false;
        } else { 
          sArgs += ", ";
        }  
        sArgs += codeArg.cCode;
        if(bArgumentSensitive)
        { //sArgSensitiveName += typeVariable[0].sArgSensitiveName;
          sArgSensitiveName += codeArg.identInfo.typeClazz.sArgSensitiveName;
          int dimension = codeArg.identInfo.getDimensionArray();
          while(dimension >0)
          { dimension -=1;
            sArgSensitiveName += "Y";  //array  
          }
        }
        if(retArgTypes != null) { retArgTypes.add(codeArg.identInfo); }
        idxArg +=1;
        sArgSeparator = ", ";
      }
    }
    if(bArgumentSensitive){ retArgSensitiveName[0] = sArgSensitiveName; }
    return sArgs;
  }  


  
  
  
  
}
