/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * @author JcHartmut: hartmut.schorrig@vishia.de
 * @version 2008-04-06  (year-month-day)
 * list of changes:
 * 2008-04-06 JcHartmut: some correction
 * 2008-03-15 JcHartmut: creation
 *
 ****************************************************************************/
package org.vishia.java2C;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.vishia.zbnf.ZbnfParseResultItem;

/**This is the superclass for {@link FirstPass} and {@link SecondPass}.
 * It contains commonly methods used in both passes and commonly data for both passes.
 * This class isn't store as instance between both passes, elsewhere the ClassData to store are stored
 * in the instance of {@link ClassData}, all the ClassData are contained in a list
 * inside {@link AllClassData}. 
 * The attributes of this class are copied from the {@link Java2C_Main} in construction 
 * and there are forgotten after running the pass.
 * 
 *  
 * 
 * @author JcHartmut
 *
 */ 
class GenerateClass
{
  /**String to generate newline and indent. */
  final private static String sNewLine = "\n                                                                              ";

  //ZbnfParseResultItem resultItemClass;
  /**A writer to file. */
  protected iWriteContent writeContent;

  /**A TreeMap of all classData sorted by classname to find all other classData.
   * This list is complete if the second path runs.
   */
  //protected final TreeMap<String, ClassData> pkgIdents;
  protected final LocalIdents fileLevelIdents;
  
  protected final GenerateFile parentGenerateFile;
 
  /**The data of this class for first and second path. A nested class will be recognized as a normal class. */
  protected ClassData classData;
  
  /**The C-name of the current translated method, used for _Stacktrace and alloc_s_BlockHeapJc. 
   * This value is set and used inside {@link SecondPass#write_methodDefinition(ZbnfParseResultItem, String, LocalIdents)}.
   */
  protected String sMethodNameCurrent;
  
  /**The interface to call a firstpass run of a needed type. */
  final RunRequiredFirstPass_ifc runRequiredFirstPass;


  
  
  
  
  
  /**constructs the class. This constructor is called from the superclass. 
   * @param writeContent The writer to the C- and H-File.
   * @param fileLevelIdents List of all known Classes while translation Java2C
   * @param runRequiredFirstPass Reference to run the first pass of a required class. 
   *        If in the first or second pass any {@link ClassData} from another class are
   *        necessary but there was not found in {@link pkgIdents}, the first pass of this
   *        required class will called recursively while processing the pass of this class. 
   */
  //protected GenerateClass(iWriteContent writeContent, TreeMap<String, ClassData> pkgIdents, RunRequiredFirstPass_ifc runRequiredFirstPass)
  protected GenerateClass(iWriteContent writeContent, GenerateFile parentGenerateFile
  , LocalIdents fileLevelIdents
  , RunRequiredFirstPass_ifc runRequiredFirstPass
  )
  { //this.sClassName = sClassName;
    //this.resultItemClass = resultItem;
    this.writeContent = writeContent;
    this.fileLevelIdents = fileLevelIdents;
    this.parentGenerateFile = parentGenerateFile;
    this.runRequiredFirstPass = runRequiredFirstPass;
  }




  /**writes the parsed description in the H-File.
   *
   * @param item description::= test_description::=
   * @throws IOException
   */
  protected void write_Description(ZbnfParseResultItem item) throws IOException
  {
    Iterator<ZbnfParseResultItem> iterDescr = item.iteratorChildren();
    if(iterDescr != null)
    { while(iterDescr.hasNext())
      { ZbnfParseResultItem resultItemDesription = iterDescr.next();
        String sSemanticDescription = resultItemDesription.getSemantic();
        if(sSemanticDescription.equals("text"))
        { String sText = resultItemDesription.getParsedString();
          writeContent.writeClassH("\n/**" + sText + "*/\n");
        }
    } }
  }


  /**Generates a variable definition with syntax of < variableDefinition>, 
   * register the variable in the LocalIdents of the class or block 
   * and registers in the list of variables to init or static variables.
   * <br>
   * The variableDefinition is used both in first and second pass of translation,
   * in the first pass there are the class variables, in the second pass there are variables
   * in block statement. But also arguments for method and constructor calls with the syntax 
   * of a <?argument> in a <code>argumentList::=...</code> are translated with this method.
   * It is the same syntax.
   * <ul>
   * <li>The {@link LocalIdents} are given per reference, it is either the instance of this {@link classData}
   *     or the instance of the block-statement. 
   * <li>If the variable is a static variable, it is registered unconditionally in the 
   *     {@link #classData}.{@link ClassData#staticVariables}. Static variables are only able at class level.
   *     Static variables may have an initialization or not.
   * <li>If the variable is non-static and it has an initial < value> or < newValue> or < newArray> 
   *     or < constArray> as ZBNF-child, than the variable is registered in the localIdents of the block or class.
   * <li>The initialization is not execute at position of definition of the variable, but
   *     <ul>
   *     <li>for class variables in the constructor.
   *     <li>for block variables after the last variable definition (C language necessity).
   *     <li>for static variables in the C-file, where the extern declaration is written with this routine
   *         also in the Headerfile. Static variables are initialized only with const values yet,
   *         the calculation of values from other values is not supported yet. If it is necessary,
   *         an extra static initializator-routine is necessary per object-file and should be organized per executable.
   *     <li>The variant of const initialization of block variables at position of variable definition
   *         isn't used, the same result is gotten by assignment of the value after all variable definitions
   *         (is there an optimization effect at C/machine-code-level?)      
   *     </ul>   
   * </ul>
   * The {@link FieldData#gen_VariableDefinition()} is called inside. The properties of the variable are stored
   * in its {@link FieldData}, which are created here from the ZBNF parse results. 
   * So the C-code-String of variable definition is able to prepare only with knowledge of the {@link FieldData}.
   * <br>
   * There are some special cases all considered here:
   * <ul>
   * <li>If it is a <code> final static </code> variable with a immediate simple assignment like
   *   <code>final static int name = 5;</code>, a <code>#define name_Classname 5</code> is produced.
   * <li>If it is a <code>final Type ... = new Type(...)</code>, an embedded struct is defined in C.
   *   That's correct, because a final reference can't be changed and the reference type is equal the
   *   instanciated type. Embedded struct constructs are opportune in C to optimize access and 
   *   memory layout of the data.
   * <li>A reference is generated either as normal forward declaration like <code>struct Type_t* ...</code>
   *   or with using an enhanced reference like <code>Type_REF ...</code>. All enhanced references 
   *   are defined together in 1 headerfile In form <pre>
   *   typedef struct NAMEREF_t { ObjectRefValuesJc refbase; struct NAME_t* ref; } NAMEREF;
   *   </pre>, see {@link referenceNeeded()} and {@link Java2C_Main.writeFileRef()}.
   * </ul>      
   *
   * @param zbnfVariableDefinition from parsers result.
   * @param zbnfDescription The description to the variable. If the variable is a method argument,
   *        than this description is gotten as a part of the method description after tag param.
   *        The description may contain java2c-tags. The following tags are respected:
   *        <ul><li><code>zeroTermString</code>
   *        <li><code>zeroTermString</code>TODO
   *        <li><code>zeroTermString</code>
   *        <li><code>zeroTermString</code>
   *        <li><code>zeroTermString</code>
   *        <li><code>zeroTermString</code>
   *        <li><code>zeroTermString</code>
   *        <li><code>zeroTermString</code>
   *        <li><code>zeroTermString</code>
   *        <li><code>zeroTermString</code>
   *        <li><code>zeroTermString</code>
   *        </ul>
   * @param localIdents The environment identifiers. Register the variable there, get types from them.
   * @param listVariablesToInit The initialization of variables is stored in this list. 
   *        The initializations should be done in C not by the variables declaration 
   *        but in another part of code. For class variables that is the constructor(s). 
   *        Therefore this parameter should be the {@link ClassData#variablesToInit} of this.
   *        But if this routine is called inside a statement block, an extra List instance
   *        for the block should given locally.
   * @param intension intension of calling 
   *        <ul>
   *        <li>A = argument list of declaration of a method or constructor. 
   *        <li>a = argument list of definition of a method or constructor.
   *        <li>b = inside statement block
   *        <li>c = class level 
   *        <li>s = static class level
   *        </ul> 
   * @return Code snipped and informations of the variables declaration.
   * @throws ParseException
   * @throws InstantiationException 
   * @throws IllegalAccessException 
   * @throws IOException 
   * @throws IllegalArgumentException 
   * @throws FileNotFoundException 
   */
  protected CCodeData gen_variableDefinition
  ( ZbnfParseResultItem zbnfVariableDefinition
  , ZbnfParseResultItem zbnfDescription
  , LocalIdents localIdents
  , List<ClassData.InitInfoVariable> listVariablesToInit
  , char intension
  )
  throws ParseException, FileNotFoundException, IllegalArgumentException, IOException, IllegalAccessException, InstantiationException
  {
    assert("Aabcs".indexOf(intension)>=0);
    /**Get all type info and the definition code of variable. */
    FieldData fieldVariable;
    /**The return instance.*/
    CCodeData codeData;
    /**generates the core of variable definition calling inside FieldData. Hint: To Debug name of variable see there!*/
    { ZbnfParseResultItem zbnfType = zbnfVariableDefinition.getChild("type");
      fieldVariable = getTypeInfo(zbnfVariableDefinition, zbnfType, zbnfDescription, localIdents
                        , "cs".indexOf(intension)>=0 ? classData: null   //declaringClass only at class level and while static class members..
                        , intension
                        );
      if(fieldVariable.getName().equals("ifc22"))
        stop();
      String code = fieldVariable.gen_VariableDefinition();
      if("Aa".indexOf(intension) <0)  //no ; at end of argument definition in argument list
      { if(fieldVariable.modeStatic != 'd'){ code += ";"  ;} //no ; at end of #define
        String sDescription = get_shortDescription(zbnfDescription);
        if(sDescription != null)
        { code += "  /*" + sDescription + "*/";
        }
      }
        
      if(fieldVariable.modeAccess=='@' || fieldVariable.modeArrayElement=='@')
      { /**A variable-definition of typeREF of the type is needed in this source.
         * Registers the type for the class.
         * The reference will be defined in the dst.h at first. */
        classData.addEnhancedRefType(fieldVariable.typeClazz.getClassIdentName());  //to define the REF-Type.
      }
      if(fieldVariable.modeAccess=='&' || fieldVariable.modeArrayElement=='&')
      { /**A variable-definition of typeMTB of the type is needed in this source.
         * Registers the type for the class.
         * The reference will be defined in the dst.h or dst.c at first. */
        classData.addMtblRefType(fieldVariable.typeClazz.getClassIdentName(), intension);  //to define the REF-Type.
      }
      codeData = new CCodeData(code, fieldVariable);
    }
    
    /**Get and store the initialization zbnf-part of the variable: */
    if( fieldVariable.modeStatic != 'd')   //not for #define ... the value is stored already in fieldVariable.
    { ZbnfParseResultItem zbnfVariablesInitialization;
      zbnfVariablesInitialization = zbnfVariableDefinition.getChild("newArray");
      if(zbnfVariablesInitialization == null)
      { zbnfVariablesInitialization = zbnfVariableDefinition.getChild("newObject");
        if(zbnfVariablesInitialization == null)
        { zbnfVariablesInitialization = zbnfVariableDefinition.getChild("value");
          if(zbnfVariablesInitialization == null)
          { zbnfVariablesInitialization = zbnfVariableDefinition.getChild("constArray");
          }
        }
      }
      boolean bStatic = "Ss".indexOf(fieldVariable.modeStatic)>=0; 
      if(bStatic || zbnfVariablesInitialization != null) //add staticVariables always, also without init value. 
      { /**There is a initialize value. */
        ClassData.InitInfoVariable variableToInit = 
          new ClassData.InitInfoVariable(fieldVariable, zbnfVariablesInitialization);
        if(bStatic){
          classData.staticVariables.add(variableToInit);  //a static initializing.
        }else if(listVariablesToInit != null){
          listVariablesToInit.add(variableToInit);        //a dynamic initializing.
        }
      }
    }
    
    /**Register the variable definition in the LocalIdents of class or block. */  
    if(localIdents != null)  //when they are null?
    { /**Put identInfo.*/
      if("cs".indexOf(intension)>=0)
      { /**at class level, class variable or static variable of the class. */
        localIdents.putClassElement(fieldVariable.getName(), fieldVariable);
      }
      else if(intension=='A')
      { /**elements in a formal argument list of declaration should not be stored, but in actual. */
      }
      else
      { localIdents.putLocalElement(fieldVariable.getName(), fieldVariable);
      }
    }
    
    return codeData;
  }
    
    
    
  
  
  /**Prepares the description from the < description>parse result item.
   * The description is shorted to one line, but only to ". ", the first sentence.
   * @param parent The item
   * @return Description guaranteed without newline.
   */
  String get_shortDescription(ZbnfParseResultItem zbnfDescription)
  { String sDescription = null;
    //ZbnfParseResultItem zbnfDescription = parent.getChild("description");
    if(zbnfDescription != null)
    { sDescription = zbnfDescription.getChild("text").getParsedString();
      int posNewline = sDescription.indexOf('\n');
      int posDot = sDescription.indexOf(". ");
      int posEnd = posDot > 0 && posDot < posNewline ? posDot : posNewline > 0 ? posNewline : -1;
      if(posEnd > 0){ sDescription = sDescription.substring(0, posEnd); }
    }
    return sDescription;
  }

  
  /**generates constants values from parse result of the syntaxterm simpleValue. 
   * It is called if a < simpleValue> is expected, but the simpleValue should or may be a constant.
   * At example generation a <code>#define NAME VALUE</code> expression in C where a 
   * <code>final static int NAME = VALUE;</code> is given in Java.  
   * 
   * @param item The part of a simpleValue what is  
   * <pre>
   * [ <""?simpleStringLiteral>
   * | <''?simpleCharLiteral>
   * | 0x<#x?hexNumber>
   * | - 0x<#x?hexNumberNegative>
   * | <#-?intNumber>[?\.]
   * | <#f?floatNumber>F
   * | <#f?doubleNumber>
   * ]</pre>
   *
   * @return null if it is not any of the named part, than it may be a non constant value, see
   *         syntax description in Java.zbnf,
   *         return the constant expression in C if it is a constant one.
   *         
   */
  protected CCodeData genConstantValue(ZbnfParseResultItem item)
  { final ClassData typeValue;
    String simpleValue;
    String sSemantic = item.getSemantic();
    if(sSemantic.equals("value"))
    { ZbnfParseResultItem zbnfChild = item.firstChild();
      return genConstantValue(zbnfChild);
    }
    else
    { if(sSemantic.equals("simpleStringLiteral"))
      { simpleValue = "\"" + item.getParsedString() + "\"";
        typeValue = Java2C_Main.singleton.standardClassData.clazz_s0;
      }
      else if(sSemantic.equals("booleanConst"))
      { simpleValue = item.getParsedString();
        typeValue = Java2C_Main.singleton.standardClassData.clazz_bool;
      }
      else if(sSemantic.equals("nullRef"))
      { simpleValue = item.getParsedString();
        typeValue = Java2C_Main.singleton.standardClassData.clazzObjectJc;
      }
      else if(sSemantic.equals("simpleCharLiteral"))
      { simpleValue = "\'" + item.getParsedString() + "\'";
        typeValue = Java2C_Main.singleton.standardClassData.clazz_char;
      }
      else if(sSemantic.equals("hexNumber"))
      { simpleValue = "0x" + Long.toHexString(item.getParsedInteger());
        typeValue = Java2C_Main.singleton.standardClassData.clazz_int32;
      }
      else if(sSemantic.equals("hexNumberNegative"))
      { simpleValue = "-0x" + Long.toHexString(item.getParsedInteger());
        typeValue = Java2C_Main.singleton.standardClassData.clazz_int32;
      }
      else if(sSemantic.equals("intNumber"))
      { simpleValue = "" + item.getParsedInteger();
        typeValue = Java2C_Main.singleton.standardClassData.clazz_int32;
      }
      else if(sSemantic.equals("floatNumber"))
      { simpleValue = "" + item.getParsedFloat()+"F";
        typeValue = Java2C_Main.singleton.standardClassData.clazz_float;
      }
      else if(sSemantic.equals("doubleNumber"))
      { simpleValue = "" + item.getParsedFloat();
        typeValue = Java2C_Main.singleton.standardClassData.clazz_double;
      }
      else
      { simpleValue = null;
        typeValue = null;
      }
      if(simpleValue == null)
      { return null;
      }
      else
      { final CCodeData codeInfo = new CCodeData(simpleValue, typeValue.classTypeInfo);
        return codeInfo;
      }
    }  
  }
  

  
  
  /**generates a String with the correct indentation. 
   * 
   * @param indent nr of indentation, as counter for nesting
   * @return "\n" and the appropriate number of spaces.
   */
  public static String genIndent(int indent)
  { return sNewLine.substring(0, 2*indent +1);
  }



  
  /**Provides the {@link ClassData} for the given type. The type is given in form of a parse result item
   * with semantic 
   * <pre>
   * type::=
   * [ [<?name>List|LinkedList|ArrayList|Iterator|Class][ \< <type?ContainerElementType> \>] 
   * | [<?name>Map|TreeMap|ArrayList|Iterator|Class][ \< <type?ContainerElementType> \>\< <type?ContainerElementType2> \>] 
   * | <envIdent?typeIdent>
   * ] 
   * [{<?typeArray> \[ \]}].
   * </pre>. 
   * Especially because the possibility of <code>&lt;envIdent?typeIdent></code> the requested type
   * may be a type inside another class (an inner class, <code>Outerclass.Innerclass</code>), 
   * so this class should be provided before, and the type should be searched there.
   * <ul>
   * <li>If the type is known, it means the {@link ClassData} with the given key are found,
   *     starting in the {@link LocalIdents} of the current context, further searching in the 
   *     {@link Java2C_Main#userTypes} and the {@link Java2C_Main#standardClassData}, 
   *     if necessary searched as inner type. It are one or some simple accesses to the appropriate 
   *     {@link LocalIdents}. 
   * <li>If the type isn't found, or the environment type (<code>&lt;envIdent?typeIdent></code>)
   *     isn't found, it is possible that this type isn't translated yet, but it is a part of the
   *     users <code>type.java</code> sources to translate. In this case a <code>type.java</code>-file
   *     should be part of input files, given with cmd line arguments <code>-i:FILE</code> or
   *     inside the <code>-if:INPFILE</code>. Therefore 
   *     {@link RunRequiredFirstPass_ifc#runRequestedFirstPass(String type)} is called.
   *     This routine either processes the parsing and first pass of translation for that file,
   *     or it parses the appropriate <code>type.stc</code> File and provides therewith
   *     the {@link ClassData}. The parsing is a deep recursively process, because other types
   *     are need too. This routine is called than recursively.
   * <li>If a file with the requested type isn't found, it may be either an not provided standard type
   *     (this is a deficiency of this translator, it may be any yet not considered Java standard class
   *     is used), or it may be an external type which is provided in the C-handwritten environment, 
   *     where a only emulating <code>type.java</code>-File in the Java sources exists. In both cases
   *     a new {@link ClassData} with attribute "external" is created.    
   * </ul>
   * In all cases a useable {@link ClassData}-instance is returned.
   *  
   * @param zbnfType The zbnf-component with syntax <code>type::=...</code>. 
   * @param localIdents The idents of this level.
   * @return The ClassData of the type.
   * @throws InstantiationException 
   * @throws IllegalAccessException 
   * @throws IOException 
   * @throws IllegalArgumentException 
   * @throws FileNotFoundException 
   * @throws ParseException 
   */
  protected ClassData getType
  ( ZbnfParseResultItem zbnfType
  , final LocalIdents localIdentsP
  ) throws FileNotFoundException, IllegalArgumentException, IOException, IllegalAccessException, InstantiationException, ParseException
  { String sRet = "";
    ClassData classDataType = null;
    ZbnfParseResultItem itemEnv;
    //ZbnfParseResultItem itemTypeIdent = zbnfType.getChild("typeIdent");
    ZbnfParseResultItem itemTypeIdent = zbnfType;
    String sType;
    final LocalIdents localIdents;
    if(itemTypeIdent != null)
    { { sType = itemTypeIdent.getChild("@name").getParsedString();
        /**NOTE: if a inner type is used line EnvType.SubType.Type, the mostly inner type is named first. */
        if(sType.equals("StringBuffer"))
          stop();
        
        if( (itemEnv=itemTypeIdent.getChild("envIdent"))!= null)
        { /**Here at first the mostly outer type is named. */
          String envIdent = itemEnv.getChild("@name").getParsedString();
          /**it should be a known type, either as inner type or as global type.
           * if it is a global type, it may be converted first.
           */
          //classDataType = localIdentsP.getTypeContingentlyRunFirstPass(envIdent, runRequiredFirstPass);  
          classDataType = localIdentsP.getType(envIdent, fileLevelIdents);  
          final LocalIdents localIdentsEnv = classDataType.classLevelIdents;
          LocalIdents localIdentsUse = localIdentsEnv;
          while((itemEnv=itemEnv.getChild("subIdent"))!= null)
          { /**Type inside the outer type. */
            String subIdent = itemEnv.getChild("@name").getParsedString();
            classDataType = localIdentsUse.getType(subIdent, null);  //fileLevelIdents = null!
            localIdentsUse = classDataType.classLevelIdents;
          }
          localIdents = localIdentsUse;
        }
        else
        { localIdents = localIdentsP; //use from given parameter
        }
      }
    }
    else
    { //containertype
      assert(false); //TODO
      sType = zbnfType.getChild("name").getParsedString();
      ZbnfParseResultItem containerElementType = zbnfType.getChild("ContainerElementType");
      localIdents = null;
    }  
    if(sType.equals("void"))
      stop();

    /**Search to appropriate ClassData. If not found, run first pass or create new ClassData for extern type. */
    if(sType.equals("Ifc"))
      stop();
    classDataType = localIdents.getType(sType, fileLevelIdents);
    if(classDataType == null)
    { /**The type is not knwon. A unknown type will be created yet and registered in userTypes.
       */
      classDataType = new ClassData("extern/", sType, sType, sType, '.', sType, '*', "+extern+");
      Java2C_Main.externalTypes.putClassType(sType, classDataType);
    }
    return classDataType;
  }


  /**Gets the info whether the type is an array type written like <code>ident[][]</code>.
   * @param zbnfType The ZBNF parse result for a type.
   * @return 0 if <code>[]</code> isn't found (parse result <code>typeArray</code> isn't present,
   *         >0 if it is a array type.
   */
  public int getDimensionsArray(ZbnfParseResultItem zbnfType)
  {
    int dimensionArray;
    /**Check if it is an array type. */
    if(zbnfType.getChild("typeArray") != null)
    { List<ZbnfParseResultItem> zbnfArray = zbnfType.listChildren("typeArray");
      dimensionArray = zbnfArray.size();
    }
    else{ dimensionArray = 0; }
    return dimensionArray;
  }
  
  
  
  
  
  
  
  /**Prepare the type properties from a given ZBNF <code>type::=</code> or adequate info.
   * The type properties are stored in a new FieldData, but without field name.
   * The type properties includes array designation and some Java2CTag.
   * @param zbnfVariable It may be null, if only the type informations should be kept.
   *                     The ZBNF result which presents a variable definition.
   *                     Some informations about the type of variable are contained in that level of parse result,
   *                     like new Array[length], final, a static initialization etc.
   * @param zbnfType The ZBNF result which presents <code>type::=</code> 
   *        or <code>nonArrayType::=</code> or <code>typeIdent::=</code>
   * @param zbnfDescription The ZBNF result which presents the description part to the type.
   *        If it is a variable definition, it is the description to the them.
   *        But if it is a argument or return type of a method, it is that part of the description
   *        of the method, which describes the param or return. 
   *        It may contain a <code><$?Java2CTag></code> to the type element.
   *        This param may be null, if no description of them is available.
   * @param localIdentsP The local visible identifiers.
   * @param intension
   * @return A new instance of FieldData containing all type informations, but without name of field
   *         and without any instance informations like fix size of array etc.
   * @throws IOException
   * @throws IllegalAccessException
   * @throws InstantiationException
   * @throws IOException 
   * @throws IllegalArgumentException 
   * @throws FileNotFoundException from called {@link #getType(ZbnfParseResultItem, LocalIdents)}.
   * @throws ParseException 
   * @since 2009-06-27
   */
  public FieldData getTypeInfo
  ( final ZbnfParseResultItem zbnfVariable 
  , final ZbnfParseResultItem zbnfType
  , final ZbnfParseResultItem zbnfDescription
  , final LocalIdents localIdentsP
  , ClassData declaringClass
  , char intension
  ) throws IllegalAccessException, InstantiationException, FileNotFoundException, IllegalArgumentException, IOException, ParseException
  { final ClassData typeClass1;
    final ClassData typeClass;
    final int dimensionArray;
  
    final String sNameField = zbnfVariable == null ? null : zbnfVariable.getChild("variableName").getParsedString();
    if(sNameField!=null && sNameField.equals("ifc3"))
        stop();
       
    if( zbnfType.getChild("va_arg") != null)
    { typeClass = Java2C_Main.singleton.standardClassData.clazz_va_argRaw; //field_int_va_arg; //clazzObjectJc;
      dimensionArray = 1;
    }
    else
    {
      typeClass1 = getType(zbnfType, localIdentsP);
      /**Test special case, String representation. */
      if(typeClass1 == CRuntimeJavalikeClassData.clazzStringJc)
      { final boolean zeroTermString = zbnfDescription != null && zbnfDescription.getChild("zeroTermString")!= null;
        /**Special case, a char* in C: */
        if(zeroTermString){ typeClass = CRuntimeJavalikeClassData.clazz_s0;}
        else { typeClass = typeClass1;}  //standard case: a StringJc in C
      }
      else
      { typeClass = typeClass1;  //standard case, not a StringJc
      }
      dimensionArray = getDimensionsArray(zbnfType);
    }
    
    char modeStatic;
    if(zbnfVariable != null){
      if(zbnfVariable.getChild("static")!= null){ 
        if(zbnfVariable.getChild("final")!= null){
          modeStatic = 'S';  //a constant value.
        } else{ modeStatic = 's'; } //static variable.
      } else{ modeStatic = '.'; }
    } else{ modeStatic = '.'; }
    
    
    char accessMode;
    String sType = typeClass.getClassCtype_s();
    if(sType.equals("StringJc"))
    { accessMode ='t';
    }
    else if(typeClass.isPrimitiveType() || typeClass.bEmbedded)
    { accessMode = '%';
    }
    else if(typeClass.isInterface() || (zbnfDescription!=null && zbnfDescription.getChild("dynamic-call")!= null))
    { accessMode = "Aab".indexOf(intension)>=0 ? '&' : '@';  //interface-reference with Method table.
    }
    else if(intension=='c' && (zbnfDescription==null || zbnfDescription.getChild("noGC")== null))
    { accessMode = '@';  //enhanced reference
    }
    else
    { accessMode = '*';  //a simple reference
    }
    
    if(dimensionArray >0)
    { return getTypeInfoArray(zbnfVariable, zbnfType, zbnfDescription, localIdentsP, sNameField, declaringClass, intension, accessMode, typeClass, modeStatic, dimensionArray);
    }
    else 
    { 
      ZbnfParseResultItem itemNewObject = zbnfVariable== null ? null : zbnfVariable.getChild("newObject");  //it may be or may null.
      if(itemNewObject != null)
      { //only at class level, test if it should be an embedded struct in C
        //TODO zbnfVariablesInitialization = itemNewObject;
        //TODO zbnfAssignment = null;
        return getTypeInfoNewObject(zbnfVariable, zbnfType, zbnfDescription, localIdentsP, sNameField, declaringClass, intension, accessMode, typeClass, modeStatic, itemNewObject);
      }  
      else 
      { //no new, no array
        //TODO zbnfVariablesInitialization = zbnfAssignment = itemVariableDefinition.getChild("value");  //it may be null
        return getTypeInfoSimple(zbnfVariable, zbnfType, zbnfDescription, localIdentsP, sNameField, declaringClass, intension, accessMode, typeClass, modeStatic, itemNewObject);
      }
    }  
  }
  
  
  
  
  FieldData getTypeInfoSimple
  ( final ZbnfParseResultItem zbnfVariable 
  , final ZbnfParseResultItem zbnfType
  , final ZbnfParseResultItem zbnfDescription
  , final LocalIdents localIdentsP
  , String sNameField
  , ClassData declaringClass
  , char intension
  , char accessMode
  , ClassData typeClass
  , char staticMode
  , final ZbnfParseResultItem itemNewObject
  ) throws FileNotFoundException, IllegalArgumentException, IOException, IllegalAccessException, InstantiationException, ParseException
  { //no new, no array
    //TODO zbnfVariablesInitialization = zbnfAssignment = itemVariableDefinition.getChild("value");  //it may be null
    if(sNameField != null && sNameField.equals("kConstant"))
      stop();
    String[] sDefineValue;
    if(staticMode == 'S'){
      CCodeData codeConstantValue;
      ZbnfParseResultItem zbnfValue;
      List<ZbnfParseResultItem> listZbnfAssignments;
      if(  intension == 's'   //static variable
        && zbnfVariable!= null
        && (zbnfValue = zbnfVariable.getChild("value"))!= null
        && (listZbnfAssignments = zbnfValue.listChildren()).size()==1   //a simple constant assignemnt
        && (codeConstantValue = genConstantValue(listZbnfAssignments.get(0))) != null
        && !codeConstantValue.identInfo.typeClazz.sClassNameC.equals("char*")  //a const char* is translated to extern const StringJc---
        ){
        staticMode = 'd';  //it is a define.
        sDefineValue = new String[1];
        sDefineValue[0] = codeConstantValue.cCode;
      } else { sDefineValue = null; }
    } else { sDefineValue = null; }   
    final ZbnfParseResultItem zbnfInstanceClass = zbnfDescription == null ? null : zbnfDescription.getChild("instanceType");
    final String sInstanceClass = zbnfInstanceClass == null ? null: zbnfInstanceClass.getParsedString();
    final ClassData instanceClass;
    if(sInstanceClass == null) { instanceClass = null; }
    else {
      //instanceClass = fileLevelIdents.getTypeContingentlyRunFirstPass(sInstanceClass, runRequiredFirstPass); 
      instanceClass = fileLevelIdents.getType(sInstanceClass, null); 
    }
    FieldData type = new FieldData(sNameField, typeClass, instanceClass, null, null, staticMode, accessMode, 0, sDefineValue /*fixArraySize*/, '.' /*modeArrayElement*/, declaringClass); 
    return type;
  }
  
  
  
  
  FieldData getTypeInfoNewObject
  ( final ZbnfParseResultItem zbnfVariable 
  , final ZbnfParseResultItem zbnfType
  , final ZbnfParseResultItem zbnfDescription
  , final LocalIdents localIdentsP
  , String sNameField
  , ClassData declaringClass
  , final char intension
  , final char accessModeP
  , ClassData typeClass
  , char staticMode
  , final ZbnfParseResultItem itemNewObject
  ) throws FileNotFoundException, IllegalArgumentException, IOException, IllegalAccessException, InstantiationException, ParseException
  { char accessMode = accessModeP;
    int sizeFixStringBuffer = 0;
    boolean bEmbeddedStruct;
    if("cb".indexOf(intension) >=0)  //class or block level
    { ZbnfParseResultItem zbnfTypeOfNew = itemNewObject.getChild("className");
      ClassData classdataNewObject = getType(zbnfTypeOfNew, localIdentsP);
      if(classdataNewObject == typeClass)
      //String sTypeNewObject = gen_EnvIdent(itemNewObject.getChild("className"), "__", idents);  //itemNewObject.getChild("className").getParsedString();
      //if(sTypeNewObject.equals(sType))
      { //it is a form Type final name = new Type(...), implement an embedded struct in C.
        String sTypeName = typeClass.getClassNameJava();
        if(sTypeName.equals("StringBuffer") || sTypeName.equals("StringBuilder")
        //if(typeVariable == Java2C_Main.singleton.standardClassData.clazzStringBufferJc
          && zbnfDescription != null && zbnfDescription.getChild("fixStringBuffer")!= null
          )
        { //The StringBuffer should have a fix direct buffer.
          final ZbnfParseResultItem zbnfArg = itemNewObject.getChild("actualArguments");
          if(zbnfArg != null)
          { final ZbnfParseResultItem zbnfValue = zbnfArg.getChild("value");
            if(zbnfValue != null)
            { final ZbnfParseResultItem zbnfNumber = zbnfValue.getChild("intNumber");
              if(zbnfNumber != null)
              {
                sizeFixStringBuffer = (int)zbnfNumber.getParsedInteger();
                accessMode = '$'; //embedded struct.
              }
            }
            
          }
        }
        if("csS".indexOf(intension)>=0)  //class variable, static class variable:
        { /**A "= new Type(...)" is found, it is an embedded struct only if the referenced is final. */
          if(zbnfVariable.getChild("final") != null)
          { accessMode = '$'; //embedded struct.
            writeContent.addIncludeH(typeClass.sFileName, "embedded type in class data");  //it is need, because the type should be known.
          }
        }
        else
        { assert(intension=='b');
          //if it is in an statement block, it is only an embedded struct, in this case in stack,
          //  only if the @java2c=stackInstance. is set.
          if(zbnfDescription != null && zbnfDescription.getChild("stackInstance")!= null)
          { writeContent.addIncludeC(typeClass.sFileName, "embedded type in block");  //it is need, because the type should be known.
            accessMode = '$'; //embedded struct.
          }
        }
      }
    }
    FieldData type = new FieldData(sNameField, typeClass, typeClass, null, null, staticMode, accessMode
        , sizeFixStringBuffer, null /*fixArraySize*/
        , sizeFixStringBuffer > 0 ? 'B' : '.' /*modeArrayElement*/
        , declaringClass ); 
    return type;

  
  }
  
  
  
  FieldData getTypeInfoArray
  ( final ZbnfParseResultItem zbnfVariable 
  , final ZbnfParseResultItem zbnfType
  , final ZbnfParseResultItem zbnfDescription
  , final LocalIdents localIdentsP
  , String sNameField
  , ClassData declaringClass
  , char intension
  , char accessMode
  , ClassData typeClass
  , char staticMode
  , int dimensionArray
  ) throws FileNotFoundException, IllegalArgumentException, IOException, IllegalAccessException, InstantiationException, ParseException
  { boolean bEmbeddedArrayElements = (zbnfDescription != null && zbnfDescription.getChild("embeddedYElements")!= null);
    boolean bSimpleArray = zbnfDescription != null && zbnfDescription.getChild("simpleArray")!= null;
    boolean bEmbeddedStruct = false;
    boolean bReference;
    boolean bFinalReference;
    char modeArrayElement = bEmbeddedArrayElements ? '$' 
                          : accessMode;  //The normal access mode of the basic type, maybe % t * or @
    String[] fixArraySizes = null;
    if(zbnfVariable != null)
    {
      ZbnfParseResultItem zbnfNewArray = zbnfVariable.getChild("newArray");  //it may be or may null.
      ZbnfParseResultItem zbnfConstArray = zbnfVariable.getChild("constArray");  //it may be or may null.
      if(zbnfNewArray != null)
      { //TODO zbnfVariablesInitialization = zbnfNewArray;
        ZbnfParseResultItem zbnfTypeOfNew = zbnfNewArray.getChild("className");
        ClassData classdataNewArray = getType(zbnfTypeOfNew, localIdentsP);
        if(classdataNewArray == typeClass)
        { //it is a form Type final name = new Type(...), implement an embedded struct in C.
          if("csS".indexOf(intension)>=0)
          { //if it is a variable of a class, static or non static,
            //it is an embedded array only if the attribute is final.  not typeClass.isFinal && 
            bEmbeddedStruct = zbnfVariable.getChild("final")!=null;
          }
          else
          { assert(intension=='b');
            //if it is in an statement block, it is only an embedded struct, in this case in stack,
            //  only if the @java2c=stackInstance. is set.
            bEmbeddedStruct = zbnfDescription != null && zbnfDescription.getChild("stackInstance")!= null;
          }
          if(bEmbeddedStruct)
          { bReference = false;
            bFinalReference = false;
            fixArraySizes = new String[dimensionArray];
            List<ZbnfParseResultItem> listSize = zbnfNewArray.listChildren("value");
            int idxArraySize = 0;
            for(ZbnfParseResultItem zbnfValueSize : listSize)
            { CCodeData codeValue = genConstantValue(zbnfValueSize);
              fixArraySizes[idxArraySize] = codeValue.cCode;
            }
          }
          else //not bEmbeddedStruct
          { bSimpleArray = false;
            fixArraySizes = null;
          }
        }
        else
        { //different Type1[] name = new Type2[]
          bSimpleArray = false; //unsued, but it have to be initzialized.
          fixArraySizes = null;
        }
        
      }
      else if(zbnfConstArray != null)
      { //TODO zbnfVariablesInitialization = zbnfConstArray;
        fixArraySizes = new String[dimensionArray];
        bEmbeddedStruct = typeClass.isFinal;
        bSimpleArray = bEmbeddedStruct;  //everytime simple if it is final, because it has only deterministic elements. 
        bReference = false;
        bFinalReference = false;
        List<ZbnfParseResultItem> listValues = zbnfConstArray.listChildren(); 
        fixArraySizes[0] = "" + listValues.size();   //only 1-dimensional, nr of values is the size 
      }
      else
      { //no ...new Type[]
        //TODO zbnfVariablesInitialization = null;
        bSimpleArray = false;
        fixArraySizes = null;
      }
                    
    }
    if(bSimpleArray)
    { accessMode = bEmbeddedStruct ? 'Q': 'P';
    }
    else
    { accessMode = bEmbeddedStruct ? 'Y': 'X';
      
    }
    FieldData type = new FieldData(sNameField, typeClass, null, null, null, staticMode, accessMode, dimensionArray, fixArraySizes, modeArrayElement, declaringClass); 
    return type;
  }
  
  
  
  

  
  
  /**It's a debug helper. The method is empty, but it is a mark to set a breakpoint. */
  protected void stop()
  { //debug
  }

}
