package org.vishia.java2C;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.Map.Entry;


/**This class contains all identifiers visible at class level or visible at a local position of code.
 * It means, visible at class level by generating the class members,
 * visible at level of a block statement including stack variable etc.
 * @author JcHartmut
 * {@link FieldData#typeClazz}
 *
 */
public class LocalIdents
{
  
  /**List of known fields of this context. */
  private final TreeMap<String, FieldData> fieldIdents;

  /**List of known types of this context. */
  //private final TreeMap<String, FieldData> typeIdents;
  private final TreeMap<String, JavaSources.ClassDataOrJavaSrcFile> typeIdents;

  /**The parent idents. */
  private final LocalIdents parent;
  
  /**Access to the file level idents. TODO: don't use for normal search from outside,
   * it should not stored here, but it should given in an extra parameter calling {@link #getTypeInfo(String, LocalIdents)}!
   * 
   */
  //private final LocalIdents fileLevelIdents;
  
  /**The ClassData for which is this member of, or null if it isn't classLevelIdents of {@link ClassData}. } */
  private final ClassData declaringClassData;
  
  /**constructs a new empty instance. 
  public LocalIdents()
  { this(Java2C_Main.singleton.userTypes);
  }
 */ 
  
  
  /**constructs a new instance with copy all parent identifier as base. 
   * This method is called especially if the identifier infos of a class is built.
   * @param parent The identifier infos of the parent scope are copied,
   *               the parent scope is not touched (changed). This parameter should be <code>null</code>
   *               while the LocalIdents of a class are created. 
   *               It should be given, if the LocalIdents of a method or statement blocks are created.
   * @param declaringClassData All identifier infos of all super and outer classes
   *               of this class are copied. This parameter should be <code>null</code>
   *               while the LocalIdents of methods or statement blocks are created.
   * @param fileLevelIdents LocalIdents of the package.              
   */
  private LocalIdents(LocalIdents parent, ClassData declaringClassData, LocalIdents fileLevelIdents, boolean hasFields)
  { if(hasFields){
      fieldIdents = new TreeMap<String, FieldData>();
      if(parent != null && parent.fieldIdents != null) { fieldIdents.putAll(parent.fieldIdents); }
    } else {
      fieldIdents = null;
      //a given parent should not have fieldIdents too!
      assert(parent == null || parent.fieldIdents == null || parent.fieldIdents.size()==0);
    }
    typeIdents = new TreeMap<String, JavaSources.ClassDataOrJavaSrcFile>();
    if(parent!=null){
      typeIdents.putAll(parent.typeIdents);  //don't copy the fileLevelIdents, only locally.
    }
    this.parent = parent;
    //this.fileLevelIdents = fileLevelIdents != null ? fileLevelIdents 
    //                 : parent != null ? parent.fileLevelIdents
    //                   : null;
    this.declaringClassData = declaringClassData; //maybe null
    if(declaringClassData != null)
    {
      if(declaringClassData.sClassNameJava.equals("Object"))
        stop();
      ClassData superClass = declaringClassData.getSuperClassData();
      while( superClass != null)
      { if(!superClass.sClassNameJava.equals("Object"))
          stop();
        Set<Entry<String,FieldData>> entrySet = superClass.classLevelIdents.fieldIdents.entrySet();
        for(Entry<String,FieldData> entry: entrySet)
        { FieldData field = entry.getValue();
          FieldData superField = new FieldData(field, 1, 0, '.');
          fieldIdents.put(superField.getName(), superField);
        }
        superClass = superClass.getSuperClassData();
      }
    }  
  }

  
  /**constructs a new instance with copy all parent identifier as base. 
   * This constructor is called especially if the identifier infos
   * of a local scope (method, statement block) are built. 
   * @param parent The identifier infos of the parent scope is copied,
   *               but the parent scope should be not touched (changed).
   */
  public LocalIdents(LocalIdents parent)
  { this(parent, null, null /*parent.fileLevelIdents*/, true);
    
  }
  

  /**constructs a new Instance for primary types. Field Identifier are not planned. 
   */
  public LocalIdents()
  { this(null, null, null, false);
  }
  

  /**constructs a new instance for a class level. 
   * All identifiers of outer and super classes of the declaring class are copied too. They are visible
   * as super identifiers or outer identifiers.
   * @param declaringClass The class where the LocalIdents are assigned to.
   * @param fileLevelIdents The LocalIdents of the package containing all package visible and commonly types.
   */
  public LocalIdents(ClassData declaringClass, LocalIdents fileLevelIdents)
  { this(null, declaringClass, fileLevelIdents, true);
  }
  
  
  
  
  
  
  /**searches the infos to the given identifier.
   * 
   * @param sIdent The identifier.
   * @return null if not found. 
   */
  public FieldData get(String sIdent)
  { return fieldIdents.get(sIdent);
  }
  
  /**searches the ClassData to the given type identifier.
   * If the type is found as a {@link JavaSrcTreeFile}, which isn't translated, its first pass is run now.
   * The first pass translates the file either, or it reads its structure (stc-)file.
   * <br><br>
   * If the type isn't found, neither as {@link JavaSrcTreeFile} nor as {@link ClassData}, 
   * this method returns <code>null</code>.
   * It may be possible that the type should be exists as a <i>unknown<i> type, than it is created, outside of this routine
   * see {@link GenerateClass#getType(org.vishia.zbnf.ZbnfParseResultItem, LocalIdents)}. 
   * It may be possible too, that the identifier may be either a type
   * or an association. If it isn't found as type, it may be found as association. Therefore this routine
   * have to be returned <code>null</code> at unknown identifiers.
   * 
   * @param sIdent The identifier.
   * @return null if the identifier is not found, elsewhere the found instance.
   * @throws IllegalArgumentException if any problem while running first pass occurs. 
   */
  public ClassData getType(String sIdent, LocalIdents fileLevelIdents)
  { final JavaSources.ClassDataOrJavaSrcFile infos = getTypeInfo(sIdent, fileLevelIdents);
    if(infos == null) return null;
    else
    { ClassData retClassData = infos.getClassData();
      if(retClassData == null){
        JavaSrcTreeFile javaFile = infos.getJavaSrc();
        try
        { /**If a identifier is found, but it has not ClassData, it is a non-translated file.
           * Than run the first pass to get the ClassData. It may be that a stc-file is read.
           */
          retClassData = Java2C_Main.singleton.runRequestedFirstPass(javaFile, sIdent);
        } 
        catch (FileNotFoundException e){ throw new IllegalArgumentException("file not found: " + javaFile.toString()); }
        catch (IOException e){ throw new IllegalArgumentException("file error: " + javaFile.toString()); }
        catch (ParseException e){ throw new IllegalArgumentException("parse exception: " + javaFile.toString()); }
        catch (InstantiationException e){ throw new IllegalArgumentException("instanciation exception: " + javaFile.toString()); }
        catch (IllegalAccessException e){ throw new IllegalArgumentException("illegal access: " + javaFile.toString()); }
        //catch (IllegalArgumentException e){ throw new IllegalArgumentException("file not found: " + javaFile.toString()); }
      }
      return retClassData;
    }
  }
  
  /**searches the infos to the given type identifier. A translation process isn't activated from here,
   * see {@link #getType(String, LocalIdents)}.
   * 
   * @param sIdent The identifier. It is possible it is a package path, 
   * at example <code>java.lang.Object</code>
   * @return null if not found. Than the type isn't known. If not null, than it may be an instance
   *   of {@link ClassData} for simple types, or it is an instance of {@link JavaSrcTreeFile}.
   *   If the file is translated already, calling {@link JavaSources.ClassDataOrJavaSrcFile#getClassData()}
   *   supplies the ClassData, if it isn't translated yet but only known as Java-File, getClassData() returns <code>null</code>.
   * @throws ParseException 
   */
  //public FieldData getType(String sIdent)
  public JavaSources.ClassDataOrJavaSrcFile getTypeInfo(final String sIdentP, LocalIdents fileLevelIdents)
  { 
    String sIdent = sIdentP.replace('/','.');  //NOTE: if '/' isn't contain, the method is optimized.
    int posDot;
    LocalIdents envIdents = this;
    while( (posDot = sIdent.indexOf('.')) >=0){
      String sEnvIdent = sIdent.substring(0, posDot);
      JavaSources.ClassDataOrJavaSrcFile envInfos = envIdents.getTypeInfo(sEnvIdent, envIdents == this ? fileLevelIdents : null);
      if(envInfos != null){
        envIdents = envInfos.getLocalIdents();
      }
      if(envIdents == null || envInfos == null){
        throw new IllegalArgumentException("package not found: " + sEnvIdent + " for " + sIdent);        
      }
      sIdent = sIdent.substring(posDot+1);
    }
        
  
    JavaSources.ClassDataOrJavaSrcFile infos;
    if(envIdents.declaringClassData !=null && envIdents.declaringClassData.sClassNameJava.equals(sIdent))
    { infos = envIdents.declaringClassData;
    }
    else
    { infos = envIdents.typeIdents.get(sIdent);
    }
    if(infos == null && fileLevelIdents != null){ // && envIdents == this && fileLevelIdents != null){
      /**search in given fileLevelIdents only if no environment ident is given.
       * At example if pkgX.class is given, the file level idents are used already to search pkgX.
       */
      infos = fileLevelIdents.typeIdents.get(sIdent);  
    }
    if( infos == null 
      //&&  envIdents != CRuntimeJavalikeClassData.singleton.stdTypes
      //&&  envIdents != Java2C_Main.externalTypes
      ){
      /**On creating standard types the fileLevelIdents are not be initialized with the standard types,
       * therefore search there.
       */
      
      infos = CRuntimeJavalikeClassData.singleton.stdTypes.typeIdents.get(sIdent); 
    }
    if(infos == null){ // &&  envIdents != Java2C_Main.externalTypes){
      /**Last not least search in external (unknwon) types. */
      infos = Java2C_Main.externalTypes.typeIdents.get(sIdent); 
    }
    return infos;  //null if not found.  
  }
  
  
  
  /**searches the infos to the given type identifier.
   * 
   * @param sIdent The identifier.
   * @return null if not found. 
   * @throws InstantiationException 
   * @throws IllegalAccessException 
   * @throws IOException 
   * @throws IllegalArgumentException 
   * @throws FileNotFoundException 
   * @throws ParseException 
   */
  //public FieldData getType(String sIdent)
  public ClassData getTypeContingentlyRunFirstPass(String sIdent, RunRequiredFirstPass_ifc runFirstPass) throws FileNotFoundException, IllegalArgumentException, IOException, IllegalAccessException, InstantiationException, ParseException
  { if(sIdent.contains("org/vishia/util/LogMessageFile.Dbg"))
      stop();
    ClassData type;
    JavaSources.ClassDataOrJavaSrcFile infos;
    infos = getTypeInfo(sIdent, null);
    if(infos == null){
      /**unknown type*/
      throw new ParseException("unknown type: " + sIdent + "\navailable type idents:" + getAllTypeIdents(),0);
      //type = runFirstPass.runRequestedFirstPass(null /*javaSrc*/, sIdent, sPkgName);
    }
    else {
      type = infos.getClassData();
      if(type == null){
        /**The type is known, but no ClassData are assigned, therefore the first pass should be run: */
        JavaSrcTreeFile javaSrc = infos.getJavaSrc();
        type = runFirstPass.runRequestedFirstPass(javaSrc, sIdent);
      }
    }
    return type;  //may be null if not found.
  }
  
  /**puts a new class element (field, attribute, reference) in the container.
   * 
   * @param sIdent The textual representation of the identifier in java code context.
   * @param sType The associated type string if it is a field or method. null if it is a type.
   * @param typeClazz The associated type class
   * @param sModifier Kind of the identifier. See table in the description of the class {@link ClassData}.
   * @param clazz The associated class data
   */
  public void putClassElement
  ( String sIdent, String sType, ClassData typeClazz, char staticMode
  , char modeAccess
  , String sModifier
  , int dimensionArray
  , String[] fixArraySizes
  , char modeArrayElement
  , ClassData clazz
  )
  { FieldData value = new FieldData(sIdent, typeClazz, null, null, null, staticMode, modeAccess, dimensionArray, fixArraySizes, modeArrayElement, clazz);
    putClassElement(sIdent, value);  
  }
  
  /**puts a new class element (field, attribute, reference) in the container.
   * @param sIdent name
   * @param identInfo all infos to the field
   */
  public void putClassElement(String sIdent, FieldData identInfo)
  { identInfo.nClassLevel = 1;
    identInfo.nOuterLevel = 1;
    declaringClassData.addField(sIdent, identInfo); //added to this.fieldIdents via putElement, but does somewhat else. 
  }
  
  
  
  /**puts a new inner class as a Type of the class in the container.
   * 
   * @param sTypeIdent The textual representation of the identifier in java code context.
   * @param sTypeNameC The type name in C, with outerclass__sTypeIdent.
   * @param typeClazz The associated type class
   * @param sModifier Kind of the identifier. See table in the description of the class {@link ClassData}.
   * @param clazz The associated class data
   * @deprecated use {@link #putClassType(String, ClassData)}
   */
  public void putClassType
  ( String sTypeIdent
  , String sTypeNameC
  , ClassData typeClazz
  , String sModifier
  , ClassData clazz
  )
  { putClassType(sTypeIdent, typeClazz);
  }
  
  
  public void putClassType
  ( String sTypeIdent
  , JavaSources.ClassDataOrJavaSrcFile typeClazz
  )
  { //FieldData value = new FieldData(sTypeNameC, typeClazz, '.', sModifier, null, clazz);
    //value.nClassLevel = 1;
    //value.nOuterLevel = 1;
    //typeIdents.put(sTypeIdent, value);
    if(sTypeIdent.equals("SetValueGenerator"))
      stop();
    typeIdents.put(sTypeIdent, typeClazz);
  }
  
  public void putClassType
  ( ClassData typeClazz
  )
  { typeIdents.put(typeClazz.getClassNameJava(), typeClazz);
  }
  
  /**Puts the class into the typeIdents, with key with and without package.
   * The class is able to found with its name only, but with its package too.
   * @param typeClazz The class.
   */
  public void putClassTypeStandard( ClassData typeClazz)
  { String sPkgClass = typeClazz.sPackage + typeClazz.sClassNameJava;
    typeIdents.put(sPkgClass, typeClazz);
    typeIdents.put(typeClazz.getClassNameJava(), typeClazz);
  }
  
  
  public void putClassTypesAll(LocalIdents parent)
  {
    for(Entry<String, JavaSources.ClassDataOrJavaSrcFile> entry: parent.typeIdents.entrySet()){
      String key = entry.getKey();
      JavaSources.ClassDataOrJavaSrcFile data = entry.getValue();
      typeIdents.put(key, data);
    }
  }
  
  
  /**puts a new stack-local element in the container.
   * @param sIdent
   * @param identInfo
   */
  public void putLocalElement(String sIdent, FieldData identInfo)
  { identInfo.nClassLevel = 0;
    identInfo.nOuterLevel = 0;
    fieldIdents.put(sIdent, identInfo);
  }
  
  /**puts a new stack-local element in the container.
   * @param sIdent
   * @param identInfo
   */
  void putElement(String sIdent, FieldData identInfo)
  { fieldIdents.put(sIdent, identInfo);
  }
  
  private final static String newLineIndent = "\n                                            ";
  
  String indent(int recursion){ return newLineIndent.substring(0, 2*recursion +1); } 
  
  
  public String writeStructOwnClassFields(int recursion)
  { String out = "";
    /*
    if(typeIdents != null)
    { out += indent(recursion) + "typeIdents {  ";
      Set<Entry<String,ClassData>> listEntries = typeIdents.entrySet();
      for(Entry<String,ClassData> field: listEntries)
      { //String sName = field.getKey();
        ClassData identInfos = field.getValue();
        out += indent(recursion+1) + "type;" + identInfos.getClassIdentName();
      }
      out += indent(recursion) + "}";
    }
    */  
    if(fieldIdents != null)
    { out += indent(recursion) + "fieldIdents {";
      Set<Entry<String,FieldData>> listEntries = fieldIdents.entrySet();
      for(Entry<String,FieldData> field: listEntries)
      { //String sName = field.getKey();
        FieldData identInfos = field.getValue();
        if(identInfos.getName().equals("fixArray"))
          stop();
        if(identInfos.nClassLevel == 1 && identInfos.nOuterLevel == 1){
          out +=  indent(recursion+1) + identInfos.writeStruct();
        }
      }
      out += indent(recursion) + "}";
    }  
    return out;
  }
  
  
  /**Returns a String with all available type idents comma separated. 
   * This String is usefull for help on error unknown type.
   */
  public String getAllTypeIdents()
  {
    String ret = "class-Level:";
    for(String sKey: typeIdents.keySet()){
      ret += " " + sKey;
    }
    //if(fileLevelIdents != null){
    //  ret += "\nfile-level:";
    //  for(String sKey: fileLevelIdents.typeIdents.keySet()){
    //  ret += " " + sKey;
    //} }
    if(Java2C_Main.singleton.standardClassData.stdTypes != this){
      ret += "\nlang-level:";
      for(String sKey: Java2C_Main.singleton.standardClassData.stdTypes.typeIdents.keySet()){
      ret += " " + sKey;
    } }
    return ret;
  }
  
  
  /**Returns an iterable list with all typeIdents.
   */
  public Set<Map.Entry<String,JavaSources.ClassDataOrJavaSrcFile>> getTypeSet()
  { return typeIdents.entrySet();
  }
  
  /**It's a debug helper. The method is empty, but it is a mark to set a breakpoint. */
  void stop(){}
}
