package org.vishia.java2C;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;
import java.util.LinkedList;
import java.util.List;

import org.vishia.mainCmd.Report;

/**An instance of this class is filled with the result of parsing an <code>*.stc</code> file
 * which contains the structure informations of a already translated file.
 * <br>
 * Such a file is parsed inside {@link #readStructToClassData(String sClassFileName)} 
 * called inside {@link #runFirstPass(File javaFile, boolean)},
 * if the destination file exists and the source <code>*.java</code> file isn't newer than the
 * already translated destination.
 * That routine parses the result and calls 
 * {@link org.vishia.zbnf.ZbnfJavaOutput#setOutputStrict(Object result, ZbnfParseResultItem, Report)}.
 * The result is this instance.
 * <br>
 * The syntax of parsing is given in the outside file <code>Java2Cstc.zbnf</code> in form:
 * <pre>
 * Structure::= J2C: structure of file-class: <$?fileName> { class <ClassData> } \e.
 * 
 * ClassData::= <*; ?nameJava> ; <$?nameC> ; 
 * \{ [ fieldIdents \{ [{ field; <field> }] \}]
 *    [ typeIdents \{ [{ <type> | class <ClassData?InnerClassData> }] \}]
 *    [ methods \{ [{ <method> }] \}]
 *    [ InnerClass \{ [{ class <ClassData?InnerClassData> }] \}]
 * \}.
 * 
 * field::= <1*?modeAccess><1*?modeArrayElement><1*?modeStatic>;
 *          [{\[ [\?\?<?arrayDimension>|<*\]?fixArraySizes>]\]}] ;
 *          <*; ?typeJava> ; <$?typeC> ; <$?name> ; 
 *          .
 *          
 * type::= type; <$?typeName>.
 * 
 * method::= method ; <*;?keyName> ; <$?javaName> ; <$?CName> ;
 *           return ; <field?returnType> 
 *           [{ param ( <field?param> ) }]
 *           .
 * </pre>.
 * All syntax/semantic elements have a representation in this class, because
 * {@link org.vishia.zbnf.ZbnfJavaOutput#setOutputStrict(Object result, ZbnfParseResultItem, Report)}.
 * sets the result of parsing directly in the instance <code>result</code> of this class.
 */ 
public class ReadStructure
{
  private final Report msg;
  
  public String sFileName;
  
  /**The package to which the classes are associated in the stc-file. */
  private String packageStc;
  
  public void set_package(String value){ packageStc = value; }
  
  /**Aggregation to the global data of java2C translator. */
  final private Java2C_Main java2c;
  
  //final private List<ClassData> listClassInFile;
  
  /**The Java source file for which this instance is created.
   * 
   */
  private final JavaSrcTreeFile javaSrc;
  
  /**The classData of the top level are to add to userTypes, inner classes are add to classLevelIdents. */
  private final LocalIdents fileLevelIdents;
    
  private final LocalIdents pkgIdents;
  
  public static class Zbnf_EnvIdent
  {
    private String[] ident;
    
    private String delim;
    
    private String name;
    
    private Zbnf_EnvIdent subIdent;
    
    public Zbnf_EnvIdent new_subIdent(){ return this; } //return new Zbnf_EnvIdent(); }
    
    public void add_subIdent(Zbnf_EnvIdent value)
    {
      //ident[0] += value + delim;
      //subIdent = value;  
    }
    
    public void set_name(String value)
    {
      ident[0] += value + delim;
    }
    
  }
  
  
  /**Subclass to pour in the result of the <code>field::=...</code> subsyntax.
   * Only some singleton instances are created:
   * <ul>
   * <li>{@link Java2C_Main.ReadStructure.Zbnf_Method#returnType1} to save the return type.
   *     It is an own instance because the return type will be processed after alle param are processed.
   * <li>{@link Java2C_Main.ReadStructure.Zbnf_Method#param} to save the current param.
   * <li>{@link Java2C_Main.ReadStructure.Zbnf_ClassData#field} to save the current field.
   * </ul>
   * They are reused, calling {@link #init()} before, if 
   * <ul>
   * <li>{@link Java2C_Main.ReadStructure.Zbnf_Method#new_param()}
   * <li>{@link Java2C_Main.ReadStructure.Zbnf_Method#new_returnType()} 
   * <li>{@link Java2C_Main.ReadStructure.Zbnf_ClassData#new_field()} 
   * </ul>
   * are called because any <code>&lt;field?...></code> were parsed with syntax 
   * <pre>
   * field::= <1*?modeAccess><1*?modeArrayElement><1*?modeStatic>;
   *          [{\[ [\?\?<?arrayDimension>|<*\]?fixArraySizes>]\]}] ;
   *          <*; ?typeJava> ; <$?typeC> ; <$?name> ; 
   * </pre>.
   * If the appropriate methods
   * <ul>
   * <li>{@link Java2C_Main.ReadStructure.Zbnf_Method#add_param(org.vishia.java2C.Java2C_Main.ReadStructure.Zbnf_Field)}
   * <li>{@link Java2C_Main.ReadStructure.Zbnf_Method#add_returnType(org.vishia.java2C.Java2C_Main.ReadStructure.Zbnf_Field)} 
   * <li>{@link Java2C_Main.ReadStructure.Zbnf_ClassData#add_field(org.vishia.java2C.Java2C_Main.ReadStructure.Zbnf_Field)} 
   * </ul>
   * are called, the content of the singleton is read out and stored 
   * in the appropriate destination data structure, that is {@link FieldData}.
   * This temporary instance is used straighten out, because the structure of syntax 
   * is not so opportune for storing in {@link FieldData} directly.
   */
  public static class Zbnf_Field
  { private char modeAccess, modeStatic, modeArrayElement;
    
    /**Stores the parsed result
     * inside {@link org.vishia.zbnf.ZbnfJavaOutput#setOutputStrict(Object result, ZbnfParseResultItem, Report)}.
     * To access the field from there via reflection, it should be public.
     */
    public String name, typeJava, instanceType;
    
    //private Zbnf_EnvIdent outerClassData;
    private String[] outerClassData = new String[1];;
    
    //private Zbnf_EnvIdent pkgType;
    private String[] pkgType = new String[1];
    
    private final Zbnf_EnvIdent envStore = new Zbnf_EnvIdent();
    
    /**If <code>&lt;*\]?fixArraySizes></code> is parsed, the String result is added to this list
     * inside {@link org.vishia.zbnf.ZbnfJavaOutput#setOutputStrict(Object result, ZbnfParseResultItem, Report)}.
     * To access the field from there via reflection, it should be public.
     */
    private List<String> listFixArraySizes = new LinkedList<String>();
    
    private int dimensionArrayOrFixSize;
    
    private Zbnf_Field()
    { modeAccess= '?'; dimensionArrayOrFixSize = 0;
    }
    
    /**Initializes the instance to use.*/
    void init()
    { modeArrayElement = modeStatic = modeAccess= '?'; 
      typeJava = ""; 
      instanceType = null;
      name = null;
      outerClassData[0] = "";
      pkgType[0] = "";
      listFixArraySizes.clear();
      dimensionArrayOrFixSize = 0;
    }

    public Zbnf_EnvIdent new_outerJava()
    { //return new Zbnf_EnvIdent(); 
      envStore.delim = ".";
      envStore.ident = outerClassData;
      outerClassData[0] = "";
      return envStore;
    }
    
    public void add_outerJava(Zbnf_EnvIdent value)
    {  envStore.ident = null;
      //outerClassData = value;
    }
    
    public Zbnf_EnvIdent new_packageType()
    { //return new Zbnf_EnvIdent(); 
      envStore.delim = "/";
      envStore.ident = pkgType;
      pkgType[0] = "";
      return envStore;
    }
    
    public void add_packageType(Zbnf_EnvIdent value)
    {  envStore.ident = null;
      //pkgType = value;
    }
    
    
    /**Sets the parsed <code>modeAccess</code>. Only the first character is used and valid. */
    public void set_modeAccess(String value){ modeAccess = value.charAt(0);}
    /**Sets the parsed <code>modeArrayElement</code>. Only the first character is used and valid. */
    public void set_modeArrayElement(String value){ modeArrayElement = value.charAt(0);}
    /**Sets the parsed <code>modeStatic</code>. Only the first character is used and valid. */
    public void set_modeStatic(String value){ modeStatic = value.charAt(0);}
    
    //public void set_name(String value){ name = value; }

    /**FromZBNF: field::=  ~~~[??&lt;?arrayDimension>]~~~ : counts the {@link #dimensionArrayOrFixSize}.  */
    public void set_arrayDimension(){ dimensionArrayOrFixSize +=1; }
    
    /**FromZBNF: field::= ~~~  <*\] ?fixArraySizes> ~~~*/
    public void add_fixArraySizes(String value){ dimensionArrayOrFixSize += 1; listFixArraySizes.add(value); }
    
    /**FromZBNF: field::= ~~~ fixSize \[ <#?fixSize> \] ~~~*/
    public void set_fixSize(int value){ dimensionArrayOrFixSize = value; }
    
  }
  
  /**Subclass to pour in the result of the <code>type::=...</code> subsyntax.
   * Only a singleton instance is created. It is reused, calling {@link #init()} before, if 
   * <ul>
   * <li>{@link Java2C_Main.ReadStructure.Zbnf_ClassData#new_type()()} 
   * </ul>
   * is called because any <code>&lt;type></code> were parsed with syntax 
   * <pre>
   * type::= type; <$?typeName>.
   * </pre>.
   * If the appropriate method
   * <ul>
   * <li>{@link Java2C_Main.ReadStructure.Zbnf_ClassData#add_type(org.vishia.java2C.Java2C_Main.ReadStructure.Zbnf_Type)} 
   * </ul>
   * is called, the content of the singleton is read out and stored 
   * in the appropriate destination data structure, that is {@link ClassData}.
   * This temporary instance is used straighten out, because the structure of syntax 
   * is not so opportune for storing in {@link ClassData} directly.
   */
  private static class Zbnf_Type
  { public String typeName;
    void init(){typeName = null; }
  }
  
  /**Subclass to pour in the result of the <code>method::=...</code> subsyntax.
   * Only a singleton instance is created. It is reused, calling {@link #init()} before, if 
   * <ul>
   * <li>{@link Java2C_Main.ReadStructure.Zbnf_ClassData#new_method()} 
   * </ul>
   * is called because any <code>&lt;method></code> were parsed with syntax 
   * <pre>
   * method::= method ; <*;?keyName> ; <$?javaName> ; <$?CName> ;
   *           return ; <field?returnType> 
   *           [{ param ( <field?param> ) }]
   * </pre>.
   * If the appropriate methods
   * <ul>
   * <li>{@link Java2C_Main.ReadStructure.Zbnf_ClassData#add_method(org.vishia.java2C.Java2C_Main.ReadStructure.Zbnf_Method)} 
   * </ul>
   * is called, the content of the singleton is read out and stored 
   * in the appropriate destination data structure, that is {@link Method}.
   * This temporary instance is used straighten out, because the structure of syntax 
   * is not so opportune for storing in {@link Method} directly.
   */
  public static class Zbnf_Method
  {
    private final ReadStructure outer;

   private String sNameJava, sCName, sNameUnambiguous, sImplementationName, sOverriddenName, sKeyName; 
    
    private String sDefPkg, sDefClass, sDefMethodUnambigous;
    
    private final List<FieldData> listParams = new LinkedList<FieldData>();
    
    private final Zbnf_Field returnType1 = new Zbnf_Field();
    
    private final Zbnf_Field param = new Zbnf_Field();
    
    private int modifier;
    
    public Zbnf_Field new_returnType(){ returnType1.init(); return returnType1; }
    public void add_returnType(Zbnf_Field dummy){} //NOTE: nothing to do, let it filled.
    
    public Zbnf_Field new_param(){ param.init(); return param; }
    
    public void add_param(Zbnf_Field param) throws FileNotFoundException, IllegalArgumentException, IOException, IllegalAccessException, InstantiationException, ParseException
    {
      //ClassData typeClass = outer.fileLevelIdents.getTypeContingentlyRunFirstPass(param.typeJava, outer.java2c);
      ClassData typeClass = outer.fileLevelIdents.getType(param.typeJava, null);
      
      FieldData identInfo = new FieldData
      ( param.name, typeClass, null, null, null, param.modeStatic, param.modeAccess
      , param.modeArrayElement, param.dimensionArrayOrFixSize, param.listFixArraySizes, null);
      listParams.add(identInfo);
      param.init();
    }
    
    public void set_keyName(String value){ sKeyName = value; }
    
    /**From ZBNF: method::= ... <$?javaName>... */
    public void set_javaName(String value){ sNameJava = value; }
    
    public void set_NameUnambiguous(String value){ sNameUnambiguous = value; }
    
    public void set_CName(String value){ sCName = value; }
    
    public void set_ImplementationName(String value){ sImplementationName = value; }
    
    public void set_OverriddenName(String value){ sOverriddenName = value; }
    
    /**From ZBNF: method::= ... mode: { ... noThCxt<?modeNoThCxt>...} */
    public void set_modeNoThCxt(){modifier |= Method.modeNoThCxt; }
    
    /**From ZBNF: method::= ... mode: { ... overrideable<?modeOverrideable>...} */
    public void set_modeOverrideable(){modifier |= Method.modeOverrideable; }
    
    /**From ZBNF: method::= ... mode: { ... returnThis<?modeReturnThis>...} */
    public void set_modeReturnThis(){modifier |= Method.modeReturnThis; }
    
    /**From ZBNF: method::= ... mode: { ... static<?modeStatic>...} */
    public void set_modeStatic(){modifier |= Method.modeStatic; }
    
    /**From ZBNF: method::= ... mode: { ... noStacktrace<?modeNoStacktrace>...} */
    public void set_modeNoStacktrace(){modifier |= Method.modeNoStacktrace; }
    
    /**From ZBNF: method::= ... <$?unambigouosName>... */
    public void set_unambigouosName(String value){ sNameUnambiguous = sNameJava + value; }
    
    /**From ZBNF: method::= ... <$?suffixName>... */
    public void set_suffixName(String value)
    { if(sNameUnambiguous == null){ sNameUnambiguous = sNameJava; } 
      sCName = sNameUnambiguous + value; 
    }
    
    
    /**From ZBNF: method::= ... [$<?classSuffixName>]... */
    public void set_classSuffixName()
    { if(sNameUnambiguous == null){ sNameUnambiguous = sNameJava; } 
      if(sCName == null){ sCName = sNameUnambiguous; } 
      sCName += outer.zbnfClassData.nameC; 
    }
    
    /**From ZBNF: method::= ... <.toLastCharIncl:/?defPkg>... */
    public void set_defPkg(String value){ sDefPkg = value; }
    
    /**From ZBNF: method::= ... <*\.?defClass>... */
    public void set_defClass(String value){ sDefClass = value; }
    
    /**From ZBNF: method::= ... <$?defMethodUnambigous>... */
    public void set_defClassAndMethodUnambigous(String value)
    { sDefMethodUnambigous = value; 
    }
    
    
    //Zbnf_Method(Java2C_Main java2c)
    //{ this.java2c = java2c;
    Zbnf_Method(ReadStructure outer)
    { this.outer = outer;
    }
    
    private void init()
    { returnType1.init();
      param.init();
      listParams.clear();
      sNameJava = sCName = sKeyName = sNameUnambiguous = sOverriddenName = sDefMethodUnambigous = null;
      sDefClass = sDefPkg = null;
      modifier = 0;
    }
  }
  
  /** @Uml=composite*/
  final Zbnf_Method zbnfMethod; 
  
  /**Subclass to pour in the result of the <code>ClassData::=...</code> subsyntax.
   * Only a singleton instance {@link Java2C_Main.ReadStructure#zbnfClassData} is created. 
   * It is reused, calling {@link #init()} before, if 
   * <ul>
   * <li>{@link Java2C_Main.ReadStructure#new_ClassData()} 
   * </ul>
   * is called because any <code>&lt;ClassData>></code> were parsed with syntax 
   * <pre>
   * ClassData::= <*; ?nameJava> ; <$?nameC> ; 
   * \{ [ fieldIdents \{ [{ field; <field> }] \}]
   *    [ typeIdents \{ [{ <type> | class <ClassData?InnerClassData> }] \}]
   *    [ methods \{ [{ <method> }] \}]
   *    [ InnerClass \{ [{ class <ClassData?InnerClassData> }] \}]
   * \}.
   * </pre>.
   * If the appropriate method
   * <ul>
   * <li>{@link Java2C_Main.ReadStructure#add_ClassData(org.vishia.java2C.Java2C_Main.ReadStructure.Zbnf_ClassData)} 
   * </ul>
   * is called, the content of the singleton is read out and stored 
   * in the appropriate destination data structure, that is {@link ClassData}.
   * This temporary instance is used straighten out, because the structure of syntax 
   * is not so opportune for storing in {@link ClassData} directly.
   */
  public static class Zbnf_ClassData
  { 
    private String sFileName; 
    
    /**Some properties. */
    private boolean bIsStaticInstance, bFinal, bInterface, bExtern, bEmbedded;
    
    private final ReadStructure outer;
        
    public String nameC, nameJava, argIdent;
    
    /**temporary list of IdentInfos to add to ClassData. */
    private final List<FieldData> listFields = new LinkedList<FieldData>();
    
    /**temporary list of Method to add to ClassData. */
    private final List<Method> listMethods = new LinkedList<Method>();
    
    /**temporary list of Method to add to ClassData. */
    private final List<String> listMethodsOverrideableC = new LinkedList<String>();
    
    /**The ClassData were set if the first field, method is set.
     * it is because the ClassData of the own class may be necessary in the own fields, methods.
     */
    private ClassData classData;
    
    private String sSuperClass;
    
    private List<String> listInterfaceClass;
    
    private final Zbnf_Field zbnf_field = new Zbnf_Field();
    
    private final Zbnf_Type zbnf_type = new Zbnf_Type();
    
    private Zbnf_ClassData zbnfInnerClassData; //NOTE: don't create initially, because it will be recursively without end.

    private final ClassData outerClass;
    
    private Zbnf_ClassData(ClassData outerClass, ReadStructure outer)
    { this.outerClass = outerClass;
      this.outer= outer;
    }
    
    /**From ZBNF: ClassData::= ... interface<?interface>... */
    public void set_interface(){ bInterface = true; }
    
    /**From ZBNF: ClassData::= ... final<?final>... */
    public void set_final(){ bFinal = true; }
    
    /**From ZBNF: ClassData::= ... extern<?extern>... */
    public void set_extern(){ bExtern = true; }
    
    /**From ZBNF: ClassData::= ... embedded<?embedded>... */
    public void set_embedded(){ bEmbedded = true; }
    
    /**From ZBNF: ClassData::= ... extends <$?superClassC>... */
    public void set_superClass(String value){ sSuperClass = value; }
    
    /**From ZBNF: ClassData::= ... implements { <?interfaceClassC> ? ,}... */
    public void add_interfaceClass(String value)
    { if(listInterfaceClass == null){ listInterfaceClass = new LinkedList<String>(); }
      listInterfaceClass.add(value); 
    }
    
    public Zbnf_Field new_field()
    { zbnf_field.init();
      return zbnf_field;
    }
    
    public void add_field(Zbnf_Field field) throws FileNotFoundException, IllegalArgumentException, IOException, IllegalAccessException, InstantiationException, ParseException
    { createClassDataIfNotDone(); 
      if(field.name.equals("sFormatter") && classData.sClassNameJava.equals("SetValueGenerator"))
        outer.java2c.stop();
      ClassData typeClass;
      if(field.outerClassData[0] != null && field.outerClassData[0].length() >0)
      { int posDelim = field.outerClassData[0].indexOf('.');
        String sOuterName = field.outerClassData[0].substring(0, posDelim);
        //ClassData outerType = outer.fileLevelIdents.getTypeContingentlyRunFirstPass(sOuterName, outer.java2c);  
        ClassData outerType = outer.fileLevelIdents.getType(sOuterName, null);  
        sOuterName = field.outerClassData[0].substring(posDelim+1);
        //Zbnf_EnvIdent itemEnv = field.outerClassData;
        //while((itemEnv = itemEnv.subIdent)!= null)
        while((posDelim = sOuterName.indexOf('.'))>0)
        { /**Type inside the outer type. */
          //String subIdent = itemEnv.name;
          String subIdent = sOuterName.substring(0, posDelim);
          outerType = outerType.classLevelIdents.getType(subIdent, null);  //fileLevelIdents=null
          sOuterName =sOuterName.substring(posDelim+1);
        }
        typeClass = outerType.classLevelIdents.getType(field.typeJava, null);
      }
      else
      { /**Gets the typeclass also as local visibility inner class (?) */
        //typeClass = classData.classLevelIdents.getTypeContingentlyRunFirstPass(field.typeJava, outer.java2c);
        typeClass = classData.classLevelIdents.getType(field.typeJava, outer.fileLevelIdents);
      }
      if(typeClass == null)
      {  outer.java2c.stop();
        typeClass = Java2C_Main.createExternalType(field.typeJava, field.pkgType[0]);
      }
      
      ClassData instanceClass = null;
      if(field.instanceType != null){
        //instanceClass = classData.classLevelIdents.getTypeContingentlyRunFirstPass(field.instanceType, outer.java2c);
        instanceClass = classData.classLevelIdents.getType(field.instanceType, outer.fileLevelIdents);
      }
      
      if(field.name.equals("ifc"))
        outer.java2c.stop();
      FieldData identInfo = new FieldData
      ( field.name, typeClass, instanceClass, null, null, field.modeStatic, field.modeAccess
      , field.modeArrayElement, field.dimensionArrayOrFixSize, field.listFixArraySizes, classData
      );
      identInfo.nClassLevel = 1;  //class elements.
      identInfo.nOuterLevel = 1;  //not copied from outer class
      listFields.add(identInfo);      
    }

    /**From ZBNF: ClassData::= ... methods \{ [{ <method> }] \}... */
    public Zbnf_Method new_method()
    { createClassDataIfNotDone(); 
      outer.zbnfMethod.init(); 
      return outer.zbnfMethod; 
    }
    
    /**From ZBNF: ClassData::= ... methods \{ [{ <method> }] \}... 
     * @throws ParseException */
    public void add_method(Zbnf_Method m) throws FileNotFoundException, IllegalArgumentException, IOException, IllegalAccessException, InstantiationException, ParseException
    {
      if(m.sCName.equals("getWay_WaySensor_s"))
        stop();
      //ClassData typeClass = classData.classLevelIdents.getTypeContingentlyRunFirstPass(m.returnType1.typeJava, outer.java2c);
      ClassData typeClass = classData.classLevelIdents.getType(m.returnType1.typeJava, outer.fileLevelIdents);
      
      FieldData returnType = new FieldData
      ( m.returnType1.name, typeClass, null, null, null, m.returnType1.modeStatic, m.returnType1.modeAccess
      , m.returnType1.modeArrayElement, m.returnType1.dimensionArrayOrFixSize, m.returnType1.listFixArraySizes, null);
      int sizeParams = m.listParams.size();
      FieldData[] params = new FieldData[sizeParams];
      int idxParam = 0;
      for(FieldData param: m.listParams)
      { params[idxParam++] = param;
      }
      /**complete non-given data: */
      if(m.sNameUnambiguous == null){ m.sNameUnambiguous = m.sNameJava; } 
      if(m.sCName == null){ m.sCName = m.sNameUnambiguous; } 
      String sKeyName = m.sNameJava + '#' + sizeParams;
      if(m.sNameJava.equals("anotherIfcmethod"))
        stop();
      classData.addMethod(m.sNameJava, m.sNameUnambiguous, m.modifier, returnType, params);
      //Method method = new Method(classData, null, sKeyName, m.sNameJava, m.sNameUnambiguous, m.modifier, returnType, params, null, "");
      //listMethods.add(method);
    }
    
    /**From ZBNF: ClassData::= ... { <$?methodOverridableC> ; }... */
    public void add_methodOverridableC(String value){ listMethodsOverrideableC.add(value); }
    
    public Zbnf_Type new_type()
    { zbnf_type.init();
      return zbnf_type; 
    }
    
    /**A Inner class is detected in ZBNF-parse-result because < ClassData?InnerClassData >.
     * Based on this, an empty instance of Zbnf_ClassData is returned to fill it. 
     * @return
     */
    public Zbnf_ClassData new_InnerClassData()
    { /**creates the outer class. */
      createClassDataIfNotDone();
      if(zbnfInnerClassData == null) 
      { zbnfInnerClassData = new Zbnf_ClassData(classData, outer); }
      zbnfInnerClassData.init(sFileName);
      return zbnfInnerClassData;
    }

    /**The inner class is filled now. Invoked from ZbnfJavaOutput, creates a new ClassData instance, fills it with given Input
     * and add it to the outer class clazzLevelEndents.
     * @param zbnfInput
     */
    public void add_InnerClassData(Zbnf_ClassData zbnfInput)
    { zbnfInput.createClassDataIfNotDone();
      for(FieldData field: zbnfInput.listFields)
      { zbnfInput.classData.classLevelIdents.putClassElement(field.getName(), field);
      }
      for(Method method: zbnfInput.listMethods)
      { //zbnfInput.classData.addMethod(method);
      }
      zbnfInput.classData.completeInheritanceWithOwnMethods();
    }

    public void add_type(Zbnf_Type value)
    { 
      
    };
    
    
    
    /**Creates the {@link ClassData} for the parsed Class if it is not existing yet.
     * @param outerClassData
     * @param superClassData
     * @param ifcClassData
     * @throws ParseException 
     * @throws InstantiationException 
     * @throws IllegalAccessException 
     * @throws IOException 
     * @throws IllegalArgumentException 
     * @throws FileNotFoundException 
     */
    private void createClassDataIfNotDone() throws IllegalArgumentException
    { if(classData == null)
      { 
        ClassData superClass = null;
        if(sSuperClass != null){
          
          try{ 
            //superClass = outer.fileLevelIdents.getTypeContingentlyRunFirstPass(sSuperClass, outer.java2c); 
            superClass = outer.fileLevelIdents.getType(sSuperClass, null); 
          }
          catch(Exception exc){
            outer.msg.reportln(Report.error, "Exception reading " + sSuperClass + ": " + exc.getMessage());
            throw new IllegalArgumentException(exc.getMessage());
            //superClass = null;
          }
        }
        
        ClassData[] ifcClasses = null;
        if(listInterfaceClass != null){
          ifcClasses = new ClassData[listInterfaceClass.size()];
          int ixIfcClasses = 0;
          for(String sIfcClass: listInterfaceClass){
            ClassData ifcClass;
            try{ 
              //ifcClass = outer.fileLevelIdents.getTypeContingentlyRunFirstPass(sIfcClass, outer.java2c); 
              ifcClass = outer.fileLevelIdents.getType(sIfcClass, null); 
            }
            catch(Exception exc){
              ifcClass = null;
              outer.msg.reportln(Report.error, "Exception reading " + sIfcClass);
            }
            ifcClasses[ixIfcClasses++] = ifcClass;
          }
        }
        
        if(nameJava.equals("ImplIfc") || nameJava.equals("ExpandedDataStruct"))
          outer.java2c.stop();
        
        classData = new ClassData(null, sFileName, outer.packageStc, nameJava, nameC
            , argIdent, 'L', this.outerClass ,superClass, ifcClasses, null
            , bIsStaticInstance, bFinal, bInterface, outer.fileLevelIdents);
        if(this.outerClass != null){
          /**An inner class: */
          this.outerClass.classLevelIdents.putClassType(classData);
        }
        else if(nameJava.equals(outer.javaSrc.getTypeName())){
          /**The public class of the file is translated: set the classData. */
          outer.javaSrc.setClassData(classData);
        } else {
          /**Additional classes in the file are found. They were unknown until now, register it. */
          outer.pkgIdents.putClassType(classData); //replace JavaSrcFile with ClassData!
          outer.fileLevelIdents.putClassType(classData.sClassNameJava, classData);
        }
      }  
    }
    
    //public void set_className(String value){ sName = value; }
    
    private void init(String sFileName)
    { this.sFileName = sFileName; nameC = nameJava = null; bIsStaticInstance = false; bFinal = false;
      bFinal = bIsStaticInstance = bInterface = bExtern = bEmbedded = false;
      sSuperClass = nameC = nameJava = null;
      if(listInterfaceClass != null) { listInterfaceClass.clear(); }
      listFields.clear();
      listMethods.clear();
      listMethodsOverrideableC.clear();
      zbnf_field.init();
      zbnf_type.init();
      if(zbnfInnerClassData != null){ zbnfInnerClassData.init(sFileName); }
      classData = null;
    }
    
    void stop(){}
  }
  
  
  /** @Uml=composite. */
  final private Zbnf_ClassData zbnfClassData; 
  
  
  //ReadStructure(Java2C_Main java2c, List<ClassData> listClassInFile, Report msg)
  ReadStructure(Java2C_Main java2c, JavaSrcTreeFile javaSrc, Report msg)
  { this.java2c = java2c;
    this.javaSrc = javaSrc;
    //this.listClassInFile = listClassInFile;
    this.msg = msg;
    this.pkgIdents = javaSrc.getPkgLevelTypes();
    this.fileLevelIdents = new LocalIdents();
    this.fileLevelIdents.putClassTypesAll(pkgIdents); //copy because used types imported in file are added.
    zbnfMethod = new Zbnf_Method(this);
    zbnfClassData = new Zbnf_ClassData(null, this);
  }

  public Zbnf_ClassData new_ClassData()
  { //ZbnfClassData obj = new ZbnfClassData();
    //obj.sFileName = sFileName;
    //return obj;
    zbnfClassData.init(sFileName);
    return zbnfClassData;
  }
  
  /**set content, able to call from ZbnfJavaOutput. */
  public void set_fileName(String value)
  { sFileName= value; }
  
  
  public void set_version(float value)
  { if(value < 0.939)
    { throw new IllegalArgumentException("Version conflict."); }
  }
  
  public void set_encoding(String value)
  {
    
  }
  
  /**Invoked from ZbnfJavaOutput, creates a new ClassData instance, fills it with given Input
   * and add it to the userTypes.
   * @param zbnfInput
   */
  public void add_ClassData(Zbnf_ClassData zbnfInput)
  { zbnfInput.createClassDataIfNotDone();
    for(FieldData field: zbnfInput.listFields)
    { if(field.getName().equals("fixArray"))
        stop();
      zbnfInput.classData.classLevelIdents.putClassElement(field.getName(), field);
    }
    for(Method method: zbnfInput.listMethods)
    { //zbnfInput.classData.addMethod(method);
    }
    //zbnfInput.classData.completeInheritanceWithOwnMethods();
    if(zbnfInput.listMethodsOverrideableC.size() >0){
      zbnfInput.classData.completeInheritanceWithListMethods(zbnfInput.listMethodsOverrideableC);
    }
  }
  
  void stop(){}
  
}

