package org.vishia.java2C.test;

/**This class contains some examples to test String functionality.
 * <br><br>
 * <b>Situation for String preparation in C:</b>
 * A String preparation in C in embedded applications is often necessary in situations of error reporting
 * or output of states. The typically choice is usage of <code>sprintf(buffer, "text-format", arguments, ...)</code>.
 * This approach have some risks which may produce errors in runtime:
 * <ul>
 * <li>The type of arguments should be matched to the designation of parts in text-format. 
 *   An error of detail is possible, the user should programmed carefully. The compiler doesn't detect errors.
 * <li>If the size of <code>buffer</code> is less, an overflow isn't detect. An overflow may occur, if a value is unexpected, 
 *   and the numbering formating produce a less longer output. <code>char*</code>-Arguments are critically too,
 *   because there length may be rate faulty because that is depending of outer code.
 * <li>If a <code>char*</code> is used as argument, and the pointer is faulty, an unexpected buffer overflow 
 *   may be produced in the <code>sprintf</code>.
 * <li>The buffer should be provided by the users environment. If the buffer is defined in Stack, 
 *   and the String is produced and used temporary, it's okay. But a software correction in calling routines,
 *   which doesn't uses to pointer to the buffer for copy, instead save the pointer, are not detected at compile time.
 * </ul>
 * The usage of <code>sprintf(...)</code> in C should implement carefully, it is sensitive. 
 * <br><br>
 * The usage of Strings in Java is some more unsensitive and simple. But the standard-Java uses the system of garbage collection
 * to accomplish this requirements. This may not able to use in all situations of embedded control-software.
 * The preparation of Strings may realized in Java in three ways:
 * <ul>
 * <li>Simple concatenations of Strings just like <code>String result = "value=" + value</code>. This variant needs a buffer,
 *   which is allocated in the heap and managed by garbage collection.
 * <li>Usage of a StringBuffer or StringBuilder. 
 *   The adequate code is <code>result.setLength(0); result.append("value=").append(value);</code>
 *   The StringBuffer-Object can be allocated as an element of any class, the memory space is allocated
 *   in the instance including the buffer itself, hence no dynamic memory allocation is necessary.
 * <li>Java knows a adequate to <code>sprintf</code> approach using <code>String.format(formatString, arguments ...)</code>. 
 *   This method is some more safety as the C-<code>sprintf</code>, because a buffer overflow is detected and the types
 *   of arguments are well known. But the method works with dynamic memory only.
 * </ul>
 * If embedded software written in Java and translated to C shouldn't used dynamic memory and garbage collection,
 * the second approach using <code>StringBuffer</code> is proper. The <code>StringBuffer</code> can be defined
 * as embedded Buffer with a fix length inside a class type. Examples using such a fix <code>StringBuffer</code>
 * allocated in the Stack too are test and shown in the method {@link #testStringBuffer()}.
 * <br><br>
 * If the garbage collector is able to use, a String concatenation is able to use. The user doesn't need to
 * provide any buffers.      
 *
 */
public class TestString
{
  
  
  /**A reference to another struct. */
  StringBuffer buffer;
  
  /**A reference to another struct, but without need of garbage collection. @java2c=noGC. */
  final StringBuffer bufferInit;
  
  
  final StringBuffer bufferEmbedded = new StringBuffer(1000);

  final static String empty = "                                     ";
  
  /**A reference to a String in memory. The String may be a reference to a constant String 
   * or to the content inside a StringBuffer. see usages. */
  String stringRef;
  
  /**Creates an embedded instance as StringBuffer with immediate space for the chars. */
  private final StringBuffer sbufferFix = new StringBuffer(250);
  
  
  TestString(StringBuffer bufferInit)
  {
    buffer = new StringBuffer(22);
    this.bufferInit = bufferInit;    
  }
  
  /**Examples for String concatenations.
   * <br><br>
   * This method tests and shows a concatenation of String using garbage collection. 
   * <br><br>
   * <b>First example: concatenation and assignment</b>: The Java-code is:
   * <pre>
    String ss = "test " + value + " miles";
    System.out.println(ss);
    this.stringRef = ss;
   * </pre>
   * A String will be built stack-locally with concatenation. Than this String will be used 
   * first as parameter for a method, than it is saved in the instance (persistent). 
   * In C it is translated to:
   * <pre>
    StringJc ss;
      ...
    StringBufferJc *_tempString1_1=null; //J2C: temporary Stringbuffer for String concatenation
      ...
    ss = 
      ( _tempString1_1 = new_StringBufferJc(-1, _thCxt)
      , setTemporary_StringBufferJc(_tempString1_1)
      , append_z_StringBufferJc(_tempString1_1, "test ", _thCxt)
      , append_I_StringBufferJc(_tempString1_1, value, _thCxt)
      , append_z_StringBufferJc(_tempString1_1, " miles", _thCxt)
      , toString_StringBufferJc(_tempString1_1)
      );
    println_s_PrintStreamJc(REFJc(out_SystemJc), ss, _thCxt);
    setRef_StringJc(&(ythis->stringRef), ss);
      ....
    activateGarbageCollectorAccess_BlockHeapJc(&_tempString1_1->base.object);
   * </pre>
   * The following principles are used:
   * <ul>
   * <li>Adequate to the implementation in Java the concatenation uses a StringBuffer internally 
   *   for the concatenation expression. The StringBuffer is allocated in heap 
   *   calling <code>new_StringBufferJc(...)</code>. 
   * <li>There is a possibility of working without allocated memory
   *   for such embedded software systems, which should not use any dynamic memory and garbage collection:
   *   The new StringBuffer can be a thread-local existing buffer. In this case only one concatenation of Strings
   *   is able to use per Thread. The content of the produced String should be copied in the next instruction.
   *   That is a restriction of programming, but it allows to work without garbage collection. Commonly
   *   a garbage collector and dynamic memory should be able to use. The CRuntimeJavalike-Library provides a solution
   *   using blocks with equal size (<code>BlockHeap</code>).
   * <li>The StringBuffer is dedicated as temporary. The reference to the StringBuffer isn't known
   *   in the users programming. Therefore the user can't work with it. Only the result String <code>ss</code>
   *   uses the buffer.
   * <li>All concatenations are implemented using <code>append...(...)</code>-methods.
   * <li>The last operation with the StringBuffer is the call of <code>toString(...)</code> to get the String.
   *   A String instance contains only the pointer to the text itself (<code>char* in Buffer</code>), 
   *   the number of chars and a few special bits. That are at maximum 2 Registers with 32 bit, 
   *   compendious in a <code>struct StringJc</code> and defined operation-system-depend 
   *   in a <code>struct OS_ValuePtr</code> to support optimal memory and register usage for the target platform.
   * <li>The String is stored and taken as parameter as value, never as a reference (not as <code>StringJc*</code>).
   *   The method <code>println(...)</code> got the String as value.
   * <li>To store a String in class variables (persistent), the method <code>setref_StringJc(...)</code> is used.
   *   This method sets the backward reference to the memory block of the block-heap, which contains
   *   the pointer to the text and sets the index of backward reference in the reference in class (<code>ythis->stringRef</code>)
   *   With this operation the block is referenced now.
   * <li>The temporary StringBufferJc is designated to managed by Garbage collection at least, before finish the routine.
   *   Because the String is referenced in this example, the garbage collector will test the reference.
   *   Elsewhere the block will be freed from garbage collector. That is typically if a String is prepared
   *   and used only locally.
   * </ul>    
   */
  void testStringConcatenation(int value)
  { String ss = "test " + value + " miles";
    System.out.println(ss);
    this.stringRef = ss;
    ss = ss + ", second";
    ss += ", third";
    ss = "forth, " + ss;
    /**A buffer in stack: @java2c=stackInstance. */
    StringBuffer buffer = new StringBuffer(40);
    buffer.append("buffer:");
    ss = buffer + ss;
    /**Test substring, indexof. */
    int pos = ss.substring(4).indexOf('i');

  }
  
  void testGarbageString()
  {
    stringRef = "other String";
    
  }
  
  /**Example for StringBuffer usage.
   * <br><br>
   * The StringBuffer {@link #bufferEmbedded} is created as an embedded part of this class structure.
   * It has a fix length, the text is part of the instance, using immediate buffer.
   * The usage of such an buffer for string preparing is a proper decision if no dynamic memory
   * should be used, and the user has the controlling and responsibility about the String instances.
   * <br>
   * The Java-code to prepare a string is:
   * <pre>
    bufferEmbedded.setLength(0);
    bufferEmbedded.append("content:").append(stringRef);
    System.out.println(bufferEmbedded);
    String s1 = bufferEmbedded.toString();
    System.out.println(s1);
    this.stringRef = s1;
   * </pre>
   * In this example the buffer is cleared, than a content is appended. The Result is used to print out
   * with given StringBuffer. Than the buffer-content is stored in a String calling the <code>toString(...)</code>-method.
   * That result is used to print out too. At last the String is stored in a class variable.
   * <br><br>
   * The resulting C-code is:
   * <pre>
    StringJc s1;
    ...
    setLength_StringBufferJc(& (ythis->bufferEmbedded.sb), 0, _thCxt);
      ( append_s_StringBufferJc(& (ythis->bufferEmbedded.sb), s0_StringJc("content:"), _thCxt)
      , append_s_StringBufferJc(& (ythis->bufferEmbedded.sb), ythis->stringRef, _thCxt)
      );
    println_sb_PrintStreamJc(REFJc(out_SystemJc), & (ythis->bufferEmbedded.sb), _thCxt);
    s1 = toString_StringBufferJc(& (ythis->bufferEmbedded.sb));
    println_s_PrintStreamJc(REFJc(out_SystemJc), s1, _thCxt);
    setRef_StringJc(&(ythis->stringRef), s1);
   * </pre>
   * The C-code follows the Java-code. The concatenation of append is resolved like described in 
   * {@link TestAllConcepts#checkConcatenationSimple()}.
   * <br><br>
   * 
   * @java2c=optimize-toString.
   */
  void testStringBuffer()
  { bufferEmbedded.setLength(0);
    bufferEmbedded.append("content:").append(stringRef);
    System.out.println(bufferEmbedded);
    String s1 = bufferEmbedded.toString();
    System.out.println(s1);
    this.stringRef = s1;
     /**The StringBuffer is allocated in BlockHeapJc, containing the max possible buffer size, 
     * this size is limited in C implementation. */
    StringBuffer sb = new StringBuffer();
    sb.setLength(0);
    sb.append("test ").append(5).append(" Stringbuffer");
    String ss = sb.toString();
    System.out.println(sb);
  }

  
  
  
  
  
  
  
  
  
  /**
   * <b>Persistence of Strings</b> 
   * The method <code>toString(...)</code> needs to turn one's attentions: Generally, a String is constant.
   * Because the buffer content of a StringBuffer may be changed after calling <code>toString(...)</code>, 
   * the text have to be copied to an independent buffer. The Java implementation do so. 
   * But this is an additional effort regarding typically applications in the fast real-time embedded world. 
   * Mostly the StringBuffer isn't change immediately, rather the got String is processed and thereby copied 
   * in the current thread like shown in the example {@link #testStringBuffer()}.
   * <br><br>
   * Thats why some methods <code>toString...(...)</code> in the CRuntimeJavalike-Library have two modes:
   * <ul>
   * <li>Compatible-mode with Java, but with more effort: <code>toString...(...)</code> creates an extra
   *   buffer for the returned String in heap.
   * <li>Small-effort-mode: <code>toString_StringBufferJc</code> references the String in the 
   *   StringBuffer-instance, no extra buffer is used. <code>toString...(...)</code> from other classes
   *   uses the thread-local Buffer, it can be used only one for one toString-result in thread, the result is to copy outside.
   *   This mode saves calculation time and prevent an extra memory allocation. 
   *   In this case the String is dedicated with a bit <code>mNonPersists__StringJc</code>.
   * </ul>    
   * Both modes are able to choice. The default is the compatible mode with Java. A method which uses 
   * the small-effort mode can be designated with a <code>@ java2c=optimize-toString</code> annotation 
   * in its description block. Than the small-effort-mode is valid for that method body. 
   * <br><br>
   * The prevention of copy a String in a allocated buffer is not only a question of calculation time effort.
   * If a routine in a fast interrupt uses a lightweight String preparation but an allocated buffer is necessary,
   * that module needs resources, which are inadequately for it. Therefore the optimize mode is recommended.
   * The Java-source should written in a form, which doesn't change a StringBuffer from which a derived 
   * temporary String is still in use. First do all with the String, than change the Buffer. 
   * Than the optimized C-code works correctly.
   * <br><br>
   * If the String is saved persistent in a class variable, it must copy to an own buffer. In this case
   * an alteration of the StringBuffer content is able to expect while the persistent String is ready to use.
   * It would be a non-obvious programming situation if the persistent String is referenced the buffer evermore.
   * Thats why the assignment using <code>setRef_StringJc(...)</code> checks whether the String is non persistent.
   * If it is so, a new StringBuffer is allocated in heap and the String is copied into. That StringBuffer
   * is only referenced by this String, and therefore it is fix and managed by garbage collection.
   * <br><br>
   * An adequate problem are given by the <code>toString(...)</code> implementations of some other Objects.
   * Because mostly the Strings will be processed immediately, at example as input for an concatenation,
   * extra buffers should be saved to minimize effort. For preparing the returned String a thread-local StringBuffer
   * is able to use. That is thread-save, but only one instance of that buffer exists per thread. 
   * This buffer have never to be referenced. Thats why the same dedication is used: <code>mNonPersists__StringJc</code>.
   * The user can produce a faulty code, a code which runs in Java because Java is save but it doesn't run in C after translation.
   * This is the disadvantage of this concept, the concession to the optimizing of calculation time and preventing
   * additional dynamically memory management. The user may write minimal other statements to get a safety code.
   * <br><br>
   * This routine is designated with the <code>@ java2c=optimize-toString.</code>, but the code is faulty for that:
   * <pre>
    bufferEmbedded.setLength(0);
    String ss = bufferEmbedded.append("content:").append(234).toString();
    bufferEmbedded.setLength(0);
    bufferEmbedded.append(" TEST ").append(ss);
    ss = bufferEmbedded.toString();
    System.out.println(ss);
   * </pre>
   * The buffer content is saved in <code>ss</code>, but after them the buffer is changed. The translated C-code is:
   * <pre>
  bool _bOptimizeStringSave;  //J2C: annotation optimize-toString
    ...
  _bOptimizeStringSave = optimizeString_ThCxt(_thCxt, true);
  { 
    StringJc ss;
    setLength_StringBufferJc(& (ythis->bufferEmbedded.sb), 0, _thCxt);
    ss = 
      ( append_s_StringBufferJc(& (ythis->bufferEmbedded.sb), s0_StringJc("content:"), _thCxt)
      , append_i_StringBufferJc(& (ythis->bufferEmbedded.sb), 234, _thCxt)
      , toString_StringBufferJc(& (ythis->bufferEmbedded.sb), _thCxt)
      );
    setLength_StringBufferJc(& (ythis->bufferEmbedded.sb), 0, _thCxt);
    
      ( append_s_StringBufferJc(& (ythis->bufferEmbedded.sb), s0_StringJc(" TEST "), _thCxt)
      , append_s_StringBufferJc(& (ythis->bufferEmbedded.sb), ss, _thCxt)
      );
    ss = toString_StringBufferJc(& (ythis->bufferEmbedded.sb), _thCxt);
    println_s_PrintStreamJc(REFJc(out_SystemJc), ss, _thCxt);
  }
  optimizeString_ThCxt(_thCxt, _bOptimizeStringSave);
   * </pre>
   * 
   * The optimizing is set in the Thread-context-data. After the routine the saved state of optimizing
   * is recovered.
   * 
   * @java2c=optimize-toString.
   */
  void testNonPersistenceOfStrings()
  {
    bufferEmbedded.setLength(0);
    String ss = bufferEmbedded.append("content:").append(234).toString();
    bufferEmbedded.setLength(0);
    bufferEmbedded.append(" TEST ").append(ss);
    ss = bufferEmbedded.toString();
    System.out.println(ss);
  }
  
  
  
  /**This routine has the same content like {@link #testNonPersistenceOfStrings()}, 
   * but the persistence is set per default,
   * no <code>@ java2c=optimize-toString</code>. is written here.
   * The result is correct. Debug it in C!
   */
  void testPersistenceOfStrings()
  {
    bufferEmbedded.setLength(0);
    String ss = bufferEmbedded.append("content:").append(234).toString();
    bufferEmbedded.setLength(0);
    bufferEmbedded.append(" TEST ").append(ss);
    ss = bufferEmbedded.toString();
    System.out.println(ss);
  }
  
  
  
  
  /**Calls the test routines.
   * 
   */
  public void testStringProcessing()
  {
    testStringConcatenation(456);
    testStringBuffer();
    testNonPersistenceOfStrings();
    testPersistenceOfStrings();
  }
  

}
